#!/bin/bash

TIMEOUT=1

EXIT_OK=0
EXIT_WARN=1
EXIT_CRIT=2
EXIT_UNKNOW=3
NOTEST=5

print_usage() 
{
	echo "" 
	echo "This snmpv2c plugin checks the hardware states on one HP server with the HP Insight System Management Agent"
	echo ""
	echo "Usage : $0 <hostname or ip> <snmp community> <check> [device]"
	echo "Usage : $0 <hostname or ip> <snmp community> <check> [device] [timeout en sec]"
	echo "Usage : $0 <hostname or ip> <snmp community> <check> [min device] [max device] <timeout en sec>"
	echo ""
	echo "The result also print <check name> <device> [details] : <state> => <description of this state>"
	echo ""
	print_commands
	exit $EXIT_UNKNOW
}

print_commands()
{
	echo "Accepted commands :"
	echo '"all"       : Realize all below tests'
	echo '"cpu"       : The Processor State'
	echo '"cntrl"     : The overall condition of controllers, and any associated logical drives, physical drives, and array accelerators. FOR TESTING WITH MULTIPLES RAID CONTROLLEURS'
	echo '"raid"      : Logical Drives Status. FOR TESTING WITH MULTIPLES RAID'
	echo '"disque"    : Status of the physical drive'
	echo '"smart"     : Physical Drive S.M.A.R.T Status.'
	echo '"pwr"       : The overall condition of the fault tolerant power supply sub-system.'
	echo '"temp"      : The Temperature sensor condition.'
	echo '"fan"       : The condition of the fan.'
}

most_critical_state()
{
	if [ "$1" -eq "$EXIT_CRIT" ] || [ "$2" -eq "$EXIT_CRIT" ]
	then return $EXIT_CRIT
	fi

	if [ "$1" -eq "$EXIT_WARN" ] || [ "$2" -eq "$EXIT_WARN" ]
	then return $EXIT_WARN
	fi

	if [ "$1" -eq "$EXIT_UNKNOW" ] || [ "$2" -eq "$EXIT_UNKNOW" ]
	then return $EXIT_UNKNOW
	fi

	if [ "$1" -eq "$EXIT_OK" ] || [ "$2" -eq "$EXIT_OK" ]
	then return $EXIT_OK
	fi
	
	return $EXIT_UNKNOW 
}

getInteger()
{
#COMMUNITY=$1
#HOST=$2
#OID=$3
    
    RET=`snmpget -t $TIMEOUT -v 2c -c $1 $2 $3`
   if [[ $? -ne 0 ]] ; then
	return 3 #Erreur de protocol
    fi

    echo $RET|grep "No Such Instance" >> /dev/null

    if [ "$?" -eq 0 ]  ; then return 1;fi #OID vide

    echo $RET|grep INTEGER >> /dev/null

    if [ "$?" -ne 0 ] ; then return 2 ;fi #Objet is not an integer 

    echo $RET|cut -d":" -f2|tr -d [:blank:]
    return 0
}

eval_cpu()
{
    STATE=$2
    case $STATE in
	1) echo "UNKNOWN: CPU $1 The status of the CPU could not be determined."; return $EXIT_UNKNOW ;;
	2) echo "OK: CPU $1 "; return $EXIT_OK;;
	3) echo "WARNING: CPU $1 Degraded => The CPU is in a pre-failure warrantee state."; return $EXIT_WARN;;
	4) echo "CRITICAL: CPU $1 Failed => The CPU is in a failed state."; return $EXIT_CRIT ;;
	5) echo "CRITICAL: CPU $1 Disabled => The CPU has been disabled during power-on-self-test."; return $EXIT_CRIT ;;
	*) echo "UNKNOWN: CPU $1 Unknown => The state of the CPU could not be determined."; return $EXIT_UNKNOW ;;
    esac
}

eval_cntrl()
{
    STATE=$2
    case $STATE in
	1) echo "UNKNOWN: cntrl $1 Other => The status of the controller could not be determined."; return $EXIT_UNKNOW ;;
	2) echo "OK: cntrl $1 => This controller, and any associated logical drives, physical drives, and array accelerators functioning normally"; return $EXIT_OK;;
	3) echo "WARNING: cntrl $1 Degraded => One or more RAIDs in controller are in degraded state"; return $EXIT_WARN;;
	4) echo "CRITICAL: cntrl $1 Failed => This controller, or any associated logical drives, physical drives, or array accelerators is in failed state"; return $EXIT_CRIT ;;
	*) echo "UNKNOWN: cntrl $1 Unknown => The state of this controller could not be determined."; return $EXIT_UNKNOW ;;
    esac
}

eval_logicalDrive()
{
    NAME="RAID $1 "
    STATE=$2
    SIZE=`getInteger $3 $4 iso.3.6.1.4.1.232.3.2.3.1.1.9.$1`
    if [ "$?" -eq 0 ] ; then let "SIZE=$SIZE / 1024"; NAME=${NAME}${SIZE}G; fi
    case $STATE in
	1) echo "UNKNOWN: ${NAME} Other => The status of the logical drive could not be determined"; return $EXIT_UNKNOW ;;
	2) echo "OK: ${NAME} => This logical drive is in normal operation mode."; return $EXIT_OK;;
	3) echo "CRITICAL: ${NAME}: Failed => More physical drives have failed than the fault tolerance mode of this logical drive can handle without data loss."; return $EXIT_CRIT;;
	4) echo "WARNING: ${NAME}: Unconfigured => This logical drive is not configured. "; return $EXIT_WARN;;
	5) echo "WARNING: ${NAME}: Recovering => The logical drive is using Interim Recovery Mode. In Interim Recovery Mode, at least one physical drive has failed, but this logical drive's fault tolerance mode lets the drive continue to operate with no data loss."; return $EXIT_WARN ;;
	6) echo "WARNING: ${NAME}: Ready Rebuild => This logical drive is ready for Automatic Data Recovery. The physical drive that failed has been replaced, but this logical drive is still operating in Interim Recovery Mode." ; return $EXIT_WARN ;;
	7) 
	    RBD=`snmpget -t $TIMEOUT -v 2c -c $3 $4 iso.3.6.1.4.1.232.3.2.3.1.1.12.$1|cut -d":" -f2|tr -d [:blank:]`
	    echo "WARNING: ${NAME} : Rebuilding $RBD% => This logical drive is currently doing Automatic Data Recovery. During Automatic Data Recovery, fault tolerance algorithms restore data to the replacement drive."; return $EXIT_WARN ;;
	8) echo "WARNING: ${NAME}: Wrong Drive => The wrong physical drive was replaced after a physical drive failure."; return $EXIT_WARN ;;
	9) echo "WARNING: ${NAME}: Bad Connect => A physical drive is not responding."; return $EXIT_WARN ;;
	10) echo "CRITICAL: ${NAME} Overheating => The drive array enclosure that contains the logical drive is overheating. The drive array is still functioning, but should be shutdown."; return $EXIT_CRIT ;;
	11) echo "CRITICAL: ${NAME} Shutdown => The drive array enclosure that contains the logical drive has overheated. The logical drive is no longer functioning."; return $EXIT_CRIT ;;
	12) echo "OK: ${NAME}: Expanding => This logical drive is currently doing Automatic Data Expansion. During Automatic Data Expansion, fault tolerance algorithms redistribute logical drive data to the newly added physical drive."; return $EXIT_OK;;
	13) echo "CRITICAL: ${NAME} Not Available => This logical drive is currently unavailable. If a logical drive is expanding and the new configuration frees additional disk space, this free space can be configured into another logical volume.  If this is done, the new volume will be set to not available."; return $EXIT_CRIT;;
	14) echo "WARNING: ${NAME}: Not Available => This logical drive is ready for Automatic Data Expansion. The logical drive is in the queue for expansion."; return $EXIT_WARNING;;
	*) echo "UNKNOWN: ${NAME} Unknown => The state of this logical drive could not be determined."; return $EXIT_UNKNOW ;;
    esac
}

eval_physicalDrive()
{
    NAME=`snmpget -t $TIMEOUT -v 2c -c $3 $4 iso.3.6.1.4.1.232.3.2.5.1.1.3.$1|cut -d'"' -f2`
    SN=`snmpget -t $TIMEOUT -v 2c -c $3 $4 iso.3.6.1.4.1.232.3.2.5.1.1.51.$1|cut -d'"' -f2|tr -s [:blank:]`
    BAY=`snmpwalk -Oqv -c $3 -v2c $4 1.3.6.1.4.1.232.3.2.5.1.1.5.$1`
    STATE=$2

    case $STATE in
	1) echo "UNKNOWN: disque $1 name $NAME SN:$SN Baie:$BAY => Other: The instrument agent does not recognize the drive.  You may need to upgrade your instrument agent and/or driver software."; return $EXIT_UNKNOW ;;
	2) echo "OK: disque $1 name $NAME SN: $SN Baie:$BAY => OK "; return $EXIT_OK;;
	3) echo "CRITICAL: disque $1 name $NAME SN: $SN Baie:$BAY => Failed: This drive is no longer operating and should be replaced."; return $EXIT_CRIT ;;
	4) echo "WARNING: $1 name $NAME SN: $SN Baie:$BAY => PredictiveFailure: This drive has a predictive failure error and should be replaced."; return $EXIT_WARN ;;
	*) echo "UNKNOWN: disque $1<$NAME>$SN Baie:$BAY => The state of this drive could not be determined."; return $EXIT_UNKNOW ;;
    esac
}

eval_smartcl()
{
    NAME=`snmpget -t $TIMEOUT -v 2c -c $3 $4 iso.3.6.1.4.1.232.3.2.5.1.1.3.$1|cut -d'"' -f2`
    SN=`snmpget -t $TIMEOUT -v 2c -c $3 $4 iso.3.6.1.4.1.232.3.2.5.1.1.51.$1|cut -d'"' -f2|tr -s [:blank:]`
    BAY=`snmpwalk -Oqv -c $3 -v2c $4 1.3.6.1.4.1.232.3.2.5.1.1.5.$1`

    STATE=$2
    case $STATE in
	1) echo "UNKNOWN: smartcl $1 name $NAME SN:$SN Baie:$BAY => Other: The agent is unable to determine he status of S.M.A.R.T predictive failure monitoring for this drive."; return $EXIT_UNKNOW ;;
	2) echo "OK: smartcl $1 name $NAME SN:$SN Baie:$BAY => OK ."; return $EXIT_OK;;
	3) echo "WARNING: smartcl $1 name $NAME SN:$SN Baie:$BAY => replaceDrive: This drive has a S.M.A.R.T predictive failure error and should be replaced."; return $EXIT_WARN ;;
	*) echo "UNKNOWN: smartcl $1 name $NAME SN:$SN Baie:$BAY => Unknown: The state of this drive could not be determined."; return $EXIT_UNKNOW ;;
    esac
}

eval_alim()
{
#$1
    STATE=$1
    case $STATE in
	1) echo "UNKNOWN: PowerSupply : Other => The status could not be determined or not present."; return $EXIT_UNKNOW ;;
	2) echo "OK: PowerSupply => This power supply is operating normally."; return $EXIT_OK;;
	3) echo "WARNING: PowerSupply => Degraded state"; return $EXIT_WARN ;;
	4) echo "CRITICAL: PowerSupply => Failed state"; return $EXIT_CRIT ;;
	*) echo "UNKNOWN: PowerSupply => The state of this power supply could not be determined."; return $EXIT_UNKNOW ;;
    esac
}

eval_temp()
{
    TEMP=`getInteger $3 $4 1.3.6.1.4.1.232.6.2.6.8.1.4.$1`
    LIMIT=`getInteger $3 $4 1.3.6.1.4.1.232.6.2.6.8.1.5.$1`
    SENSOR_TYPE=`getInteger $3 $4 1.3.6.1.4.1.232.6.2.6.8.1.3.$1`
    case "$SENSOR_TYPE" in
	1) SENSOR_DESCR="other";;
	2) SENSOR_DESCR="unknown";;
	3) SENSOR_DESCR="system";;
	4) SENSOR_DESCR="systemBoard";;
	5) SENSOR_DESCR="ioBoard";;
	6) SENSOR_DESCR="cpu";;
	7) SENSOR_DESCR="memory";;
	8) SENSOR_DESCR="storage";;
	9) SENSOR_DESCR="removableMedia";;
	10) SENSOR_DESCR="powerSupply" ;;
	11) SENSOR_DESCR="ambient";;
	12) SENSOR_DESCR="chassis";;
	13) SENSOR_DESCR="bridgeCard";;
    esac

    STATE=$2
    case $STATE in
	1) echo "UNKNOWN: sensor $1 $SENSOR_DESCR: Other => Temperature could not be determined."; return $EXIT_UNKNOW ;;
	2) echo "OK: sensor $1 $SENSOR_DESCR=$TEMP Limit = $LIMIT. OK"; return $EXIT_OK;;
	3) echo "WARNING: sensor $1 $SENSOR_DESCR=$TEMP Limit = $LIMIT. Degraded => The temperature sensor is outside of normal operating range."; return $EXIT_WARN ;;
	4) echo "CRITICAL: sensor $1 $SENSOR_DESCR=$TEMP Limit = $LIMIT. Failed =>"; return $EXIT_CRIT ;;
	*) echo "UNKNOWN: sensor $1 $SENSOR_DESCR => Temperature could not be determined."; return $EXIT_UNKNOW ;;
    esac
}

eval_fan()
{
    LOC=`getInteger $3 $4 1.3.6.1.4.1.232.6.2.6.7.1.3.$1`
    case $LOC in
	1) LOCATION="other";;
	2) LOCATION="unknown";;
	3) LOCATION="system";;
	4) LOCATION="systemBoard";;
	5) LOCATION="ioBoard";;
	6) LOCATION="cpu";;
	7) LOCATION="memory";;
	8) LOCATION="storage";;
	9) LOCATION="removableMedia";;
	10) LOCATION="powerSupply";;
	11) LOCATION="ambient";;
	12) LOCATION="chassis";;
	13) LOCATION="bridgeCard";;
	*) LOCATION="unknown";;
    esac

    SP=`getInteger $3 $4 1.3.6.1.4.1.232.6.2.6.7.1.6.$1`
    case $SP in
	1) SPEED="other";;
	2) SPEED="normal";;
	3) SPEED="high";;
	*) SPEED="unknown";;
    esac

    STATE=$2
    case $STATE in
	1) echo "OK: fan $1 $LOCATION=$SPEED : OK"; return $EXIT_OK;;
	2) echo "OK: fan $1 $LOCATION=$SPEED : OK"; return $EXIT_OK;;
	3) echo "WARNING: fan $1 $LOCATION=$SPEED : degraded => A redundant fan is not operating."; return $EXIT_WARN ;;
	4) echo "CRITICAL: fan $1 $LOCATION=$SPEED : failed =>   A non-redundant fan is not operating."; return $EXIT_CRIT ;;
	*) echo "UNKNOWN: fan $1 => Temperature could not be determined."; return $EXIT_UNKNOW ;;
    esac
}

check_fan()
{
    I=0
    ITEMS_STATE=$NOTEST		
		  
	CHASSISINDEXS=$(snmpwalk -Oqv -c $1 -v2c $2  1.3.6.1.4.1.232.6.2.6.7.1.1 2> /dev/null)
	CHASSISINDEXS=$(echo $CHASSISINDEXS|tr ' ' '\n' | uniq)
	for CHASSISINDEX in ${CHASSISINDEXS}
	do

	    OUTPUT1=$(snmpwalk -Oqv -c $1 -v2c $2 1.3.6.1.4.1.232.6.2.6.7.1.2.$CHASSISINDEX 2> /dev/null)
	    for INDEX in ${OUTPUT1}
	    do
		let I++
		if [ "$I" -lt "$3" ] ; then continue; fi
		if [ "$I" -gt "$4" ] 
		then 
		    if [ "$ITEMS_STATE" -eq "$NOTEST" ] ; then ITEMS_STATE=$EXIT_UNKNOW; fi
		    return $ITEMS_STATE
		fi

		RET=`getInteger $1 $2 1.3.6.1.4.1.232.6.2.6.7.1.9.$CHASSISINDEX.$INDEX`
		ERROR=$?
		OID=${CHASSISINDEX}.$INDEX
		case $ERROR in
		    0) eval_fan $OID $RET $1 $2; ITEM_STATE=$? ;;  ##=> for MODIFY
		    1) return $ITEMS_STATE;;
		    2) echo "Invalid OID value" ; return $EXIT_UNKNOW ;;
		    3) echo "Fan $I : SNMP Protocol Error"; ITEM_STATE=$EXIT_UNKNOW ;;  ##=> for MODIFY
		esac

		most_critical_state $ITEMS_STATE $ITEM_STATE
		ITEMS_STATE=$?
	    done
	done
    return $ITEMS_STATE
}

check_temp()
{
    I=0
    ITEMS_STATE=$NOTEST
	CHASSISINDEXS=$(snmpwalk -Oqv -c $1 -v2c $2  1.3.6.1.4.1.232.6.2.6.8.1.1 2> /dev/null)
	CHASSISINDEXS=$(echo $CHASSISINDEXS|tr ' ' '\n' | uniq)
	for CHASSISINDEX in ${CHASSISINDEXS}
	do

	    OUTPUT1=$(snmpwalk -Oqv -c $1 -v2c $2 1.3.6.1.4.1.232.6.2.6.8.1.2.$CHASSISINDEX 2> /dev/null)
	    for INDEX in ${OUTPUT1}
	    do
		let I++
		if [ "$I" -lt "$3" ] ; then continue; fi
		if [ "$I" -gt "$4" ] 
		then 
		    if [ "$ITEMS_STATE" -eq "$NOTEST" ] ; then ITEMS_STATE=$EXIT_UNKNOW; fi
		    return $ITEMS_STATE
		fi

		RET=`getInteger $1 $2 1.3.6.1.4.1.232.6.2.6.8.1.6.$CHASSISINDEX.$INDEX`
		ERROR=$?
		OID=${CHASSISINDEX}.$INDEX
		case $ERROR in
		    0) eval_temp $OID $RET $1 $2; ITEM_STATE=$? ;;  ##=> for MODIFY
		    1) return $ITEMS_STATE;;
		    2) echo "Invalid OID value" ; return $EXIT_UNKNOW ;;
		    3) echo "Sensor $I : SNMP Protocol Error"; ITEM_STATE=$EXIT_UNKNOW ;;  ##=> for MODIFY
		esac

		most_critical_state $ITEMS_STATE $ITEM_STATE
		ITEMS_STATE=$?
	    done
	done
    return $ITEMS_STATE
}

check_alim()
{
#	RET=`getInteger $1 $2 iso.3.6.1.4.1.232.6.2.9.1.1`
#	ERROR=$?
#
#	case $ERROR in
#	0) eval_alim $RET; return $? ;; 
#	1) echo "No value "; return $EXIT_UNKNOWN ;;
#	2) echo "Invalid OID value" ; return $EXIT_UNKNOW ;;
#	3) echo "PowerSupply : SNMP Protocol Error"; return $EXIT_UNKNOW ;; 
#	esac
    GLOBAL_PSU_STATUS=$(snmpwalk -v 2c -c $1 $2 .1.3.6.1.4.1.232.6.2.9.1 | cut -f4 -d ' ')
    PSU_CONDITIONS=$(snmpwalk -v 2c -c $1 $2 .1.3.6.1.4.1.232.6.2.9.3.1.4)
    
    case "$GLOBAL_PSU_STATUS" in
	1) 
	    EXIT_STRING="UNKNOWN: Checking power supplies is not supported for $2! Make sure SNMP is properly configured and that all Insight Management Agents are installed.\n"
	    PSU_EXIT_STATUS=3
	    ;;
	2|3|4)
	    if [ $GLOBAL_PSU_STATUS -eq 3 ]; then
		EXIT_STRING="WARNING:"
		PSU_EXIT_STATUS=1
	    else 
		if [ $GLOBAL_PSU_STATUS -eq 2 ]; then
		    EXIT_STRING="OK:"
		    PSU_EXIT_STATUS=0
		else
		    EXIT_STRING="CRITICAL:"
		    PSU_EXIT_STATUS=2
		fi
	    fi

			# First, filter our conditions for only PSUs in degraded state
	    PSUs_AFFECTED=$(echo "$PSU_CONDITIONS" | egrep "INTEGER: 2|INTEGER: 3|INTEGER: 4")
			# Determine how many PSUs we have that are degraded
	    NUM_PSUs_AFFECTED=$(echo "$PSUs_AFFECTED" | wc -l)

	    CURRENT_PSU=1
			# For each degraded PSU, output it's status
	    while [ $CURRENT_PSU -le $NUM_PSUs_AFFECTED ]; do
				# 	We need to differentiate if it's status was degraded or failed
		PSU_STATUS=$(echo "$PSUs_AFFECTED" | cut -f4 -d ' ' | awk NR==$CURRENT_PSU)
				# 	And finally, we can get the error code for the current power supply
		PSU_ERROR_CODE=$(snmpwalk -v 2c -c $1 $2 .1.3.6.1.4.1.232.6.2.9.3.1.5.1.$CURRENT_PSU | cut -f4 -d ' ')
		case "$PSU_ERROR_CODE" in
		    1) PSU_ERROR_DESCR="No Error"
			EXIT_STRING="OK:"
			PSU_EXIT_STATUS=0
			;;
		    2) PSU_ERROR_DESCR="2 (generalFailure)";;
		    3) PSU_ERROR_DESCR="3 (bistFailure)";;
		    4) PSU_ERROR_DESCR="4 (fanFailure)";;
		    5) PSU_ERROR_DESCR="5 (tempFailure)";;
		    6) PSU_ERROR_DESCR="6 (interlockOpen)";;
		    7) PSU_ERROR_DESCR="7 (epromFailed)";;
		    8) PSU_ERROR_DESCR="8 (vrefFailed)";;
		    9) PSU_ERROR_DESCR="9 (dacFailed)";;
		    10) PSU_ERROR_DESCR="10 (ramTestFailed)";;
		    11) PSU_ERROR_DESCR="11 (voltageChannelFailed)";;
		    12) PSU_ERROR_DESCR="12 (orringdiodeFailed)";;
		    13) PSU_ERROR_DESCR="13 (brownOut)";;
		    14) PSU_ERROR_DESCR="14 (giveupOnStartup)";;
		    15) PSU_ERROR_DESCR="15 (nvramInvalid)";;
		    16) PSU_ERROR_DESCR="16 (calibrationTableInvalid)";;
		esac

				# Report whether the status was failed/degraded
		if [ $PSU_STATUS -eq 3 ]; then
		    PSU_STATUS_TEXT="is degraded"
		else
		    PSU_STATUS_TEXT="has failed"
		fi
		
				# Determine redundant status of current psu
		PSU_REDUNDANCY=$(snmpwalk -v 2c -c $1 $2 .1.3.6.1.4.1.232.6.2.9.3.1.9.1.$CURRENT_PSU | cut -f4 -d ' ')
		case "$PSU_REDUNDANCY" in
		    2) PSU_REDUNDANCY_TEXT="Non-redundant";;
		    3) PSU_REDUNDANCY_TEXT="Redundant";;
		    *) PSU_REDUNDANCY_TEXT=""
		esac
				# Make sure we list all PSU's affected, separating each by a ","
		if [ $PSU_STATUS == 2 ]; then
		    echo "$EXIT_STRING Power Supply $PSU_REDUNDANCY_TEXT $CURRENT_PSU ==> No error."
		else
		    echo "$EXIT_STRING Power Supply $PSU_REDUNDANCY_TEXT $CURRENT_PSU $PSU_STATUS_TEXT with error $PSU_ERROR_DESCR."
		fi
		
		CURRENT_PSU=$[ $CURRENT_PSU+1 ]
		
	    done
	    ;;
    esac
	#echo "$EXIT_STRING"
    return 	$PSU_EXIT_STATUS
}

check_smartcl()
{
    I=-1
    ITEMS_STATE=$NOTEST
CNTRLINDEXS=$(snmpwalk -Oqv -c $1 -v2c $2  1.3.6.1.4.1.232.3.2.2.1.1.1 2> /dev/null)
  ERROR=$?
        if [ "$ERROR" -eq "1" ]
        then
                echo "NO SNMP RESPONSE"
                ITEM_STATE=$EXIT_UNKNOWN
	fi
for CNTRLINDEX in ${CNTRLINDEXS}
do
    OUTPUT1=$(snmpwalk -Oqv -c $1 -v2c $2 1.3.6.1.4.1.232.3.2.5.1.1.2 2> /dev/null)
    for INDEX in ${OUTPUT1}
    do
	let I++
	if [ "$I" -lt "$3" ] ; then continue; fi
	if [ "$I" -gt "$4" ] 
	then 
	    if [ "$ITEMS_STATE" -eq "$NOTEST" ] ; then ITEMS_STATE=$EXIT_UNKNOW; fi
	    return $ITEMS_STATE
	fi

	RET=`getInteger $1 $2 iso.3.6.1.4.1.232.3.2.5.1.1.57.${CNTRLINDEX}.$INDEX`
	ERROR=$?
	OID=${CNTRLINDEX}.$INDEX
	case $ERROR in
	    0) eval_smartcl $OID $RET $1 $2; ITEM_STATE=$? ;;  ##=> for MODIFY
	    1) return $ITEMS_STATE;;
	    2) echo "Invalid OID value" ; return $EXIT_UNKNOW ;;
	  #  3) echo "Smartcl $I : SNMP Protocol Error"; ITEM_STATE=$EXIT_UNKNOW ;;  ##=> for MODIFY
	esac

	most_critical_state $ITEMS_STATE $ITEM_STATE
	ITEMS_STATE=$?
    done
done
    return $ITEMS_STATE
}

check_physicalDrive()
{
    	I=-1
    	ITEMS_STATE=$NOTEST
	CNTRLINDEXS=$(snmpwalk -Oqv -c $1 -v2c $2  1.3.6.1.4.1.232.3.2.2.1.1.1 2> /dev/null)
	ERROR=$?
	if [ "$ERROR" -eq "1" ]
	then
		echo "NO SNMP RESPONSE"
		ITEM_STATE=$EXIT_UNKNOWN
	fi

	for CNTRLINDEX in ${CNTRLINDEXS}
	do
	    OUTPUT1=$(snmpwalk -Oqv -c $1 -v2c $2 1.3.6.1.4.1.232.3.2.5.1.1.2 2> /dev/null)
	    for INDEX in ${OUTPUT1}
	    do
		let I++
		if [ "$I" -lt "$3" ] ; then continue; fi
		if [ "$I" -gt "$4" ] 
		then 
		    if [ "$ITEMS_STATE" -eq "$NOTEST" ] ; then ITEMS_STATE=$EXIT_UNKNOW; fi
		    return $ITEMS_STATE
		fi

		RET=`getInteger $1 $2 iso.3.6.1.4.1.232.3.2.5.1.1.6.${CNTRLINDEX}.$INDEX`
		ERROR=$?
		#echo "RET ^$RET^$ERROR^||||"
		OID=${CNTRLINDEX}.$INDEX
		case $ERROR in
		    0) eval_physicalDrive $OID $RET $1 $2; ITEM_STATE=$? ;;  ##=> for MODIFY
		    1) return $ITEMS_STATE;;
		    2) echo "Invalid OID value" ; return $EXIT_UNKNOW ;;
		  #  3) echo "Disque $I : SNMP Protocol Error"; ITEM_STATE=$EXIT_UNKNOW ;;  ##=> for MODIFY
		esac
		most_critical_state $ITEMS_STATE $ITEM_STATE
		ITEMS_STATE=$?
	    done
	done
	    return $ITEMS_STATE
}

check_logicalDrive() #//Apparemment on commance a compter a partir de 1 et non 0 pour ces OIDs
{ 
    I=-1
    J=1
    ITEMS_STATE=$NOTEST
	CNTRLINDEXS=$(snmpwalk -Oqv -c $1 -v2c $2  1.3.6.1.4.1.232.3.2.3.1.1.1 2> /dev/null)
	for CNTRLINDEX in ${CNTRLINDEXS}
	do
	    OUTPUT1=$(snmpwalk -Oqv -c $1 -v2c $2 1.3.6.1.4.1.232.3.2.3.1.1.2 2> /dev/null)
	    for INDEX in ${OUTPUT1}
	    do
		let I++
		if [ "$I" -lt "$3" ] ; then continue; fi
		if [ "$I" -gt "$4" ] 
		then 
		    if [ "$ITEMS_STATE" -eq "$NOTEST" ] ; then ITEMS_STATE=$EXIT_UNKNOW; fi
		    return $ITEMS_STATE
		fi

		RET=`getInteger $1 $2 iso.3.6.1.4.1.232.3.2.3.1.1.4.${CNTRLINDEX}.$INDEX`
		ERROR=$?
		OID=${CNTRLINDEX}.$INDEX
		case $ERROR in
		    0) eval_logicalDrive $OID $RET $1 $2; ITEM_STATE=$? ;;  ##=> for MODIFY
		    1) if [ "$J" -gt 3 ] ; then return $ITEMS_STATE; fi ; let J++ ; continue ;;
		    2) if [ "$J" -gt 3 ] ; then return $ITEMS_STATE; fi ; let J++ ; continue ;;
		    3) echo "RAID $I : SNMP Protocol Error"; ITEM_STATE=$EXIT_UNKNOW ;;  ##=> for MODIFY
		esac
		J=1
		most_critical_state $ITEMS_STATE $ITEM_STATE
		ITEMS_STATE=$?
	    done
done
    return $ITEMS_STATE
}

check_cpus()
{
    I=-1
    ITEMS_STATE=$NOTEST
    OUTPUT1=$(snmpwalk -Oqv -c $1 -v2c $2 1.3.6.1.4.1.232.1.2.2.1.1.1 2> /dev/null)
    for INDEX in ${OUTPUT1}
    do
	let I++
	if [ "$I" -lt "$3" ] ; then continue; fi
	if [ "$I" -gt "$4" ] 
	then 
	    if [ "$ITEMS_STATE" -eq "$NOTEST" ] ; then ITEMS_STATE=$EXIT_UNKNOW; fi
	    return $ITEMS_STATE
	fi
	
	RET=`getInteger $1 $2 iso.3.6.1.4.1.232.1.2.2.1.1.6.$INDEX`
	ERROR=$?

	case $ERROR in
	    0) eval_cpu $INDEX $RET; ITEM_STATE=$? ;;  ##=> for MODIFY
	    1) return $ITEMS_STATE;;
	    2) echo "Invalid OID value" ; return $EXIT_UNKNOW ;;
	    3) echo "CPU $I : SNMP Protocol Error"; ITEM_STATE=$EXIT_UNKNOW ;;  ##=> for MODIFY
	esac

	most_critical_state $ITEMS_STATE $ITEM_STATE
	ITEMS_STATE=$?
    done
    return $ITEMS_STATE
}

check_cntrl()
{
    I=-1
    ITEMS_STATE=$NOTEST
    OUTPUT1=$(snmpwalk -Oqv -c $1 -v2c $2 1.3.6.1.4.1.232.3.2.2.1.1.1 2> /dev/null)
    for INDEX in ${OUTPUT1}
    do
	let I++
	if [ "$I" -lt "$3" ] ; then continue; fi
	if [ "$I" -gt "$4" ] 
	then 
	    if [ "$ITEMS_STATE" -eq "$NOTEST" ] ; then ITEMS_STATE=$EXIT_UNKNOW; fi
	    return $ITEMS_STATE
	fi
	
	RET=`getInteger $1 $2 iso.3.6.1.4.1.232.3.2.2.1.1.6.$INDEX`
	ERROR=$?

	case $ERROR in
	    0) eval_cntrl $INDEX $RET; ITEM_STATE=$?;;  ##=> for MODIFY
	    1) return $ITEMS_STATE;;
	    2) echo "Invalid OID value" ; return $EXIT_UNKNOW ;;
	    3) echo "SLOT $I : SNMP Protocol Error"; ITEM_STATE=$EXIT_UNKNOW ;;  ##=> for MODIFY
	esac
	most_critical_state $ITEMS_STATE $ITEM_STATE
	ITEMS_STATE=$?

    done
    return $ITEMS_STATE
}

if [ $# -lt 3 ] ; then
    print_usage
fi

case $# in 
    3) MIN=0; MAX=100 ;;	
    4) MIN=$4; MAX=$4 ;;
    5) MIN=$4; MAX=$4; TIMEOUT=$5 ;;
    6) MIN=$4; MAX=$5; TIMEOUT=$6 ;;
esac

#MIN=$4
#MAX=$5

#if [ -z "$5" ]; then
#	MAX=$MIN
#fi

#if [ -z "$4" ]; then
#	MIN=0; MAX=100
#fi

HOST=$1
COMMUNITY=$2
COMMAND=`echo $3|tr '[:lower:]' '[:upper:]'`

GEN_STATE=$NOTEST

if [ "$COMMAND" == "CPU" ] || [ "$COMMAND" == "ALL" ]
then 
    check_cpus $COMMUNITY $HOST $MIN $MAX
    most_critical_state $GEN_STATE $?
    GEN_STATE=$?
fi

if [ "$COMMAND" == "CNTRL" ] || [ "$COMMAND" == "ALL" ]
then 
    check_cntrl $COMMUNITY $HOST $MIN $MAX
    most_critical_state $GEN_STATE $?
    GEN_STATE=$?
fi

if [ "$COMMAND" == "RAID" ] || [ "$COMMAND" == "ALL" ]
then 
    check_logicalDrive $COMMUNITY $HOST $MIN $MAX
    most_critical_state $GEN_STATE $?
    GEN_STATE=$?
fi

if [ "$COMMAND" == "DISQUE" ] || [ "$COMMAND" == "ALL" ]
then 
    check_physicalDrive $COMMUNITY $HOST $MIN $MAX
    most_critical_state $GEN_STATE $?
    GEN_STATE=$?
fi

if [ "$COMMAND" == "SMART" ] || [ "$COMMAND" == "ALL" ]
then 
    check_smartcl $COMMUNITY $HOST $MIN $MAX
    most_critical_state $GEN_STATE $?
    GEN_STATE=$?
fi

if [ "$COMMAND" == "PWR" ] || [ "$COMMAND" == "ALL" ]
then 
    check_alim $COMMUNITY $HOST $MIN $MAX
    most_critical_state $GEN_STATE $?
    GEN_STATE=$?
fi

if [ "$COMMAND" == "TEMP" ] || [ "$COMMAND" == "ALL" ]
then 
    check_temp $COMMUNITY $HOST $MIN $MAX
    most_critical_state $GEN_STATE $?
    GEN_STATE=$?
fi

if [ "$COMMAND" == "FAN" ] || [ "$COMMAND" == "ALL" ]
then 
    check_fan $COMMUNITY $HOST $MIN $MAX
    most_critical_state $GEN_STATE $?
    GEN_STATE=$?
fi
if [ "$COMMAND" == "DISKHEALTH" ]
then 
    SERVER_MODEL_NUMBER=$(snmpwalk -v 2c -c $2 $1 .1.3.6.1.4.1.232.2.2.4.2 | cut -f2 -d ':' | tr -d '"')
    check_logicalDrive $COMMUNITY $HOST $MIN $MAX
    most_critical_state $GEN_STATE $?
    GEN_STATE=$?
    check_cntrl $COMMUNITY $HOST $MIN $MAX
    most_critical_state $GEN_STATE $?
    GEN_STATE=$?
    check_physicalDrive $COMMUNITY $HOST $MIN $MAX
    most_critical_state $GEN_STATE $?
    GEN_STATE=$?	
    check_smartcl $COMMUNITY $HOST $MIN $MAX
    most_critical_state $GEN_STATE $?
    GEN_STATE=$?
fi
if [ "$COMMAND" == "SYSTEMHEALTH" ]
then 
    SERVER_MODEL_NUMBER=$(snmpwalk -v 2c -c $2 $1 .1.3.6.1.4.1.232.2.2.4.2 | cut -f2 -d ':' | tr -d '"')
    echo "Model:$SERVER_MODEL_NUMBER"
    check_fan $COMMUNITY $HOST $MIN $MAX
    most_critical_state $GEN_STATE $?
    GEN_STATE=$?
    check_alim $COMMUNITY $HOST $MIN $MAX
    most_critical_state $GEN_STATE $?
    GEN_STATE=$?	
    check_cpus $COMMUNITY $HOST $MIN $MAX
    most_critical_state $GEN_STATE $?
    GEN_STATE=$?

fi

if [ "$NOTEST" -eq "$GEN_STATE" ] 
then  
    GEN_STATE=$EXIT_UNKNOW
    echo -e "Command "$3" doesn't exist"
    print_commands
fi 

exit $GEN_STATE
