#!/usr/bin/perl -w

# basic requirements
use strict;
use Getopt::Long;
use Net::SNMP;

my $progname= "check_hwg_esia.pl";
my $snmp_version  = "1";
# options
my $opt_community = "public";
my $opt_host= "127.0.0.1";
my $opt_sensor;
my $opt_contact;
my $opt_warning=25;
my $opt_critical=30;
my $opt_calibre=0;
my $opt_help;
my $opt_man;
# oids
my $enterprise = '.1.3.6.1.4.1.21796';
my $deviceid = '.1.3.6.1.2.1.1.1.0';
my @oids;
my $response;
my $output;
my $sensor_id;
my $return=0;

Getopt::Long::Configure('bundling');
GetOptions(
	'h'       => \$opt_help,
	'help'    => \$opt_help,
	'H=s'     => \$opt_host,
	'C=s',    => \$opt_community,
	'S=n',    => \$opt_sensor,
	'I',      => \$opt_contact,
	'w=s'     => \$opt_warning,
	'c=s'     => \$opt_critical,
	'd=s'	  => \$opt_calibre,
	'man'     => \$opt_man,
	'V'	  => \$snmp_version
  );

# Help
if ($opt_help || $opt_man) {
	print_help();
}

# Check if all needed options present.
unless ( $opt_host && ( $opt_sensor || $opt_contact ) && $opt_community ) 
{
	print "\nNot enough options specified !\n";
	print "Display manual with option -h or --help\n Do you forget the sensor ID?\n";
    exit(3);
}
else
{
	# Open SNMP Session
	my ( $session, $error ) = Net::SNMP->session(
		-hostname  => $opt_host,
		-community => $opt_community,
		-port      => 161,
		-version   => $snmp_version,		
	);

	# SNMP Session failed
	if ( !defined($session) ) {
		print "Unknown error : $error";
		exit(3);
	}

	# Si un ID sensor est bien défini
	if (defined $opt_sensor)
	{
		$response = $session->get_request($deviceid);
		my $device = $response->{$deviceid};
		


		# setting per device OIDs for sensor values
		if ($device =~ m/Poseidon/i) 
		{
			# get the sensor ID
			$response = $session->get_table($enterprise.".3.3.3.1.8");	
		
			#On va récupérer le bon sensor grâce à son ID		
			foreach my $sensor (keys %$response) 
			{
				if($response->{$sensor} eq $opt_sensor) 
				{
					$sensor_id = (split(/\./, $sensor))[-1];
				};	
			}
			push(@oids, $enterprise.'.3.3.3.1.2.'.$sensor_id);			# POSEIDON-MIB::sensName
			push(@oids, $enterprise.'.3.3.3.1.4.'.$sensor_id);			# POSEIDON-MIB::sensState
			push(@oids, $enterprise.'.3.3.3.1.5.'.$sensor_id);			# POSEIDON-MIB::sensString
			push(@oids, $enterprise.'.3.3.3.1.6.'.$sensor_id);			# POSEIDON-MIB::sensValue
			push(@oids, $enterprise.'.3.3.3.1.9.'.$sensor_id);			# POSEIDON-MIB::sensUnit
			push(@oids, $enterprise.'.3.3.99.1.2.1.6.'.$sensor_id);		# POSEIDON-MIB::sensLimitMin
			push(@oids, $enterprise.'.3.3.99.1.2.1.7.'.$sensor_id);		# POSEIDON-MIB::sensLimitMax
		} 
		elsif ($device =~ m/STE2/i) 
		{
			# get the sensor ID
			$response = $session->get_table("1.3.6.1.4.1.21796.4.9.3.1.2");	
		
			#On va récupérer le bon sensor grâce à son ID		
			foreach my $sensor (keys %$response) 
			{
				if($response->{$sensor} eq $opt_sensor) 
				{
					$sensor_id = (split(/\./, $sensor))[-1];
				};	
			}
			push(@oids, $enterprise.'.4.9.3.1.2.'.$sensor_id);			# POSEIDON-MIB::sensName
			push(@oids, $enterprise.'.4.9.3.1.3.'.$sensor_id);			# POSEIDON-MIB::sensState
			push(@oids, $enterprise.'.4.9.3.1.4.'.$sensor_id);			# POSEIDON-MIB::sensString
			push(@oids, $enterprise.'.4.9.3.1.5.'.$sensor_id);			# POSEIDON-MIB::sensValue
		}
		elsif ($device =~ m/STE/i) 
		{
			# get the sensor ID
			$response = $session->get_table($enterprise.".4.1.3.1.8");	
		
			#On va récupérer le bon sensor grâce à son ID		
			foreach my $sensor (keys %$response) 
			{
				if($response->{$sensor} eq $opt_sensor) 
				{
					$sensor_id = (split(/\./, $sensor))[-1];
				};	
			}
			push(@oids, $enterprise.'.4.1.3.1.2.'.$sensor_id);			# POSEIDON-MIB::sensName
			push(@oids, $enterprise.'.4.1.3.1.3.'.$sensor_id);			# POSEIDON-MIB::sensState
			push(@oids, $enterprise.'.4.1.3.1.4.'.$sensor_id);			# POSEIDON-MIB::sensString
			push(@oids, $enterprise.'.4.1.3.1.5.'.$sensor_id);			# POSEIDON-MIB::sensValue
		} else {
			print "ERROR: device not supported";
			$session->close();
			exit(3);
		}

		# Récupérer les valeurs
		$response = $session->get_request(-varbindlist => \@oids) 
			or die "ERROR while getting Sensor values";
	
		# ICI CHECK VALEUR WARNING, CRITICAL (-w -c)
		if(defined $opt_warning || defined $opt_critical)
		{
			my $value = (split / /,$response->{$oids[2]})[0];
			if($opt_calibre != 0)
			{
				$value += $opt_calibre;
			}

			if($value >= $opt_warning)
			{
				if($value >= $opt_critical)
				{
					$output .= "CRITICAL $value";
					$return = 2;
				}	
				else
				{
					$output .= "WARNING $value";
					$return = 1;
				}			
			}			
			else
			{
				print "OK $value";
				$return = 0;
			}
		}

		# setting the output string
		$output .= " Sensor: ".$response->{$oids[0]}.", ";
		$output .= "Value: ".$response->{$oids[2]};
		$output .= "| $response->{$oids[0]}=".eval($response->{$oids[3]}/10).";";

	}

	# Contact
	if (defined $opt_contact) 
	{
		# get the contact values
		#REINITIALISE LE TABLEAU OIDS
		@oids=();
		push(@oids, $enterprise.'.3.3.1.1.3.1');	# POSEIDON-MIB::inpName
		push(@oids, $enterprise.'.3.3.1.1.2.1');	# POSEIDON-MIB::inpValue
		push(@oids, $enterprise.'.3.3.1.1.4.1');	# POSEIDON-MIB::inpAlarmSetup
		push(@oids, $enterprise.'.3.3.1.1.5.1');	# POSEIDON-MIB::inpAlarmState

		# getting the values from the device
		$response = $session->get_request(-varbindlist => \@oids) 
                        or print "ERROR: Sensor ID not found\n";

		print $response->{$oids[1]}."\n";

		# setting the output string
		if(($response->{$oids[1]}) == 1)
		{
			$output .= "\nCRITICAL : Porte ouverte";
			$return = 2;
		}		
		else
		{		
			$output .= "\nPorte fermee";
		}
	}

	$session->close();

	# Afficher les données récoltées
	print $output."\n";	
	exit($return);
}

sub print_help {
	print "Manuel $progname : \n\n";
	print "Ce script a comme objectif de récupérer les données d'un appareil HwG.\n";
	print "Les options disponibles sont :\n";
	print "\t '-C' : (Communauté) spécifie la communauté.\n"; 
	print "\t '-H' : Spécifie l'adresse IP.\n"; 
	print "\t '-S' : (sensor) Spécifie l'identifiant de la sonde.\n";
	print "\t '-I' : Précise si la porte est ouverte ou fermee\n";
	print "\t '-w' : (warning) Spécifie une valeur d'avertissement.\n";
	print "\t '-c' : (critical) Spécifie une valeur critique.\n";
	print "\t '-V' : (version) Spécifie la version SNMP.\n";
	print "\t '--help' : Affiche le manuel du script.\n";
	print "\t '-h' : Affiche le manuel du script.\n\n\n";
}

__END__
