#!/usr/bin/perl
#
use strict;
use warnings;
use lib "/usr/local/esia/plugins"  ;
use vars qw($PROGNAME);
use strict;
use Net::SNMP;
use Getopt::Long;
use Data::Dumper;

my $TIMEOUT = 50;
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);

# Just in case of problems, let's not hang Nagios
$SIG{'ALRM'} = sub {
	print ("ERROR: Plugin took too long to complete (alarm)\n");
	exit $ERRORS{"UNKNOWN"};
};
my @priorities=(
                $ERRORS{CRITICAL},
                $ERRORS{WARNING},
                $ERRORS{UNKNOWN},
                $ERRORS{OK});
sub priority_state{
        my($a,$b) = @_;
        if( $a == $b ) { return $a; } #pas besoin d'aller dans la boucles si ils sont identiques

        foreach my $v (@priorities){
                if ( $v == $a || $v == $b ) { return $v; }
        }
} 
sub getsnmpOID {
    my($snmp,$oid) = @_;
	my $result = $snmp->get_request("$oid");
	return $$result{"$oid"};
}
alarm($TIMEOUT);

$PROGNAME = "check_snmp_ups.pl";
sub print_help ();
sub print_usage ();
use POSIX qw(floor);

my ($opt_h,$opt_V);
my $community = "public";
my $snmp_version = 1;
my ($hostname);
my $lineLoadWarning=70;
my $lineLoadCritical=80;
my $timecritical=60;
my $return =0;
my $oid_sysObjectOID = "1.3.6.1.2.1.1.2.0";
my $oid_upsAlarm = "1.3.6.1.2.1.33.1.6.2.1";
use Getopt::Long;
&Getopt::Long::config('bundling');
GetOptions(
	"V"   => \$opt_V,	"version"    => \$opt_V,
	"h"   => \$opt_h,	"help"       => \$opt_h,
	"C=s" => \$community,	"community=s" => \$community,
	"H=s" => \$hostname,	"hostname=s" => \$hostname,
	"v=i" => \$snmp_version,"snmp_version=i" => \$snmp_version,
	"w=i" => \$lineLoadWarning,	"warning=i" => \$lineLoadWarning,
	"c=i" => \$lineLoadCritical,	"critical=i" => \$lineLoadCritical,
	"tc=i" => \$timecritical,	"timecritical=i" => \$timecritical,
);
# -h & --help print help
if ($opt_h) { print_help(); exit $ERRORS{'OK'}; }
# -V & --version print version
if ($opt_V) { print_revision($PROGNAME,'$Revision: 0.1 $ '); exit $ERRORS{'OK'}; }
# Invalid hostname print usage
#if (!utils::is_hostname($hostname)) { print_usage(); exit $ERRORS{'UNKNOWN'}; }


my ($snmp, $snmperror);
if ($snmp_version == 2) {
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv2c',
		-community => $community,
        -timeout => 3,
	);
} elsif ($snmp_version == 3) {
	my ($v3_username,$v3_password,$v3_protocol,$v3_priv_passphrase,$v3_priv_protocol) = split(":",$community);
	my @auth = ();
	if (defined($v3_password)) { push(@auth,($v3_password =~ /^0x/) ? 'authkey' : 'authpassword',$v3_password); }
	if (defined($v3_protocol)) { push(@auth,'authprotocol',$v3_protocol); }
	if (defined($v3_priv_passphrase)) { push(@auth,($v3_priv_passphrase =~ /^0x/) ? 'privkey' : 'privpassword',$v3_priv_passphrase); }
	if (defined($v3_priv_protocol)) { push(@auth,'privprotocol',$v3_priv_protocol); }

	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv3',
		-username => $v3_username,
		@auth
	);
} else {
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv1',
		-community => $community
	);
}

if (!defined($snmp)) {
	print ("UNKNOWN - SNMP error: $snmperror\n");
	exit $ERRORS{'UNKNOWN'};
}

my $state = 'UNKNOWN';
my $output = "UPS status retrieval failed.";

#on interroge en SNMP pour récupérer les informations 
my $result = $snmp->get_request("$oid_sysObjectOID");
#si l'host ne répond pas on blocke le script
if (!defined($result)) {
	if($snmp_version == 2)
	{
		($snmp, $snmperror) = Net::SNMP->session(
			-hostname => $hostname,
			-version => 'snmpv1',
			-community => $community
		);
		$result = $snmp->get_request("$oid_sysObjectOID");
		if (!defined($result)) 
		{
	   		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
	   		$snmp->close;
	  		 exit $ERRORS{"UNKNOWN"};
		}
	}
	else
	{
		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
		$snmp->close;
		 exit $ERRORS{"UNKNOWN"};
	}
}
my $Sysoid_value = $$result{"$oid_sysObjectOID"};
if($Sysoid_value =~/3\.6\.1\.4\.1\.([0-9]*)/)
{
    $Sysoid_value = $1;
}
#affichage pour les status
my %upsBatteryStatuses = (
		1 => "unknown(1)",
		2 => "batteryNormal(2)",
		3 => "batteryLow(3)",
		4 => "batteryDepleted(4)"
	);
# Begin plugin check code
{

	my $upsBatteryStatus = "1.3.6.1.2.1.33.1.2.1.0";
	my $upsOutputSource = "1.3.6.1.2.1.33.1.4.1.0";
	my $upsEstimatedMinutesRemaining ="1.3.6.1.2.1.33.1.2.3.0";

	my $upsOutputLineIndex 	 ="1.3.6.1.2.1.33.1.4.4.1.1";
	my $upsOutputVoltage 	 ="1.3.6.1.2.1.33.1.4.4.1.2.";
	my $upsOutputCurrent 	 ="1.3.6.1.2.1.33.1.4.4.1.3.";
	my $upsOutputPower 	 ="1.3.6.1.2.1.33.1.4.4.1.4.";
	my $upsOutputPercentLoad ="1.3.6.1.2.1.33.1.4.4.1.5.";

	my %upsOutputSourcesMessage = (
		"1" => "other(1)",
		"2" => "none(2)",
		"3" => "normal(3)",
		"4" => "bypass(4)",
		"5" => "battery(5)",
		"6" => "booster(6)",
		"7" => "reducer(7)"
	);
	my %upsOutputAlarmMessage = (
        "1.3.6.1.2.1.33.1.6.3.1" => "Battery is bad",
        "1.3.6.1.2.1.33.1.6.3.10" => "Bypass is bad",
        "1.3.6.1.2.1.33.1.6.3.11" => "Output off as resquested ",
        "1.3.6.1.2.1.33.1.6.3.12" => "UPS off as resquested",
        "1.3.6.1.2.1.33.1.6.3.13" => "Charger is failed",
        "1.3.6.1.2.1.33.1.6.3.14" => "Output is Off",
        "1.3.6.1.2.1.33.1.6.3.15" => "System is Off",
        "1.3.6.1.2.1.33.1.6.3.16" => "Fan is failed",
        "1.3.6.1.2.1.33.1.6.3.17" => "Fuse is failed",
        "1.3.6.1.2.1.33.1.6.3.18" => "General failure",
        "1.3.6.1.2.1.33.1.6.3.19" => "Diagnostic failed ",
        "1.3.6.1.2.1.33.1.6.3.2" => "Battery is a warning state",
        "1.3.6.1.2.1.33.1.6.3.20" => "Communication lost",
        "1.3.6.1.2.1.33.1.6.3.21" => "Awaiting power",
        "1.3.6.1.2.1.33.1.6.3.22" => "Shutdown pending",
        "1.3.6.1.2.1.33.1.6.3.23" => "Shutdown iminent",
        "1.3.6.1.2.1.33.1.6.3.24" => "Test in progress",
        "1.3.6.1.2.1.33.1.6.3.3" => "Battery low",
        "1.3.6.1.2.1.33.1.6.3.4" => "Battery is depleted",
        "1.3.6.1.2.1.33.1.6.3.5" => "Temperature alarm",
        "1.3.6.1.2.1.33.1.6.3.6" => "Input power alarm",
        "1.3.6.1.2.1.33.1.6.3.7" => "Ouput power alarm",
        "1.3.6.1.2.1.33.1.6.3.8" => "Overload",
        "1.3.6.1.2.1.33.1.6.3.9" => "By pass",
	);
	my %upsOutputAlarmStatus = (
        "1.3.6.1.2.1.33.1.6.3.1" => "CRITICAL",
        "1.3.6.1.2.1.33.1.6.3.10" => "CRITICAL",
        "1.3.6.1.2.1.33.1.6.3.11" => "WARNING",
        "1.3.6.1.2.1.33.1.6.3.12" => "WARNING",
        "1.3.6.1.2.1.33.1.6.3.13" => "CRITICAL",
        "1.3.6.1.2.1.33.1.6.3.14" => "WARNING",
        "1.3.6.1.2.1.33.1.6.3.15" => "WARNING",
        "1.3.6.1.2.1.33.1.6.3.16" => "CRITICAL",
        "1.3.6.1.2.1.33.1.6.3.17" => "CRITICAL",
        "1.3.6.1.2.1.33.1.6.3.18" => "CRITICAL",
        "1.3.6.1.2.1.33.1.6.3.19" => "WARNING",
        "1.3.6.1.2.1.33.1.6.3.2" => "WARNING",
        "1.3.6.1.2.1.33.1.6.3.20" => "WARNING",
        "1.3.6.1.2.1.33.1.6.3.21" => "WARNING",
        "1.3.6.1.2.1.33.1.6.3.22" => "WARNING",
        "1.3.6.1.2.1.33.1.6.3.23" => "WARNING",
        "1.3.6.1.2.1.33.1.6.3.24" => "WARNING",
        "1.3.6.1.2.1.33.1.6.3.3" => "WARNING",
        "1.3.6.1.2.1.33.1.6.3.4" => "CRITICAL",
        "1.3.6.1.2.1.33.1.6.3.5" => "CRITICAL",
        "1.3.6.1.2.1.33.1.6.3.6" => "CRITICAL",
        "1.3.6.1.2.1.33.1.6.3.7" => "CRITICAL",
        "1.3.6.1.2.1.33.1.6.3.8" => "CRITICAL",
        "1.3.6.1.2.1.33.1.6.3.9" => "CRITICAL",
	);
    $upsEstimatedMinutesRemaining =  getsnmpOID($snmp,$upsEstimatedMinutesRemaining);
	$upsBatteryStatus =  getsnmpOID($snmp,$upsBatteryStatus);
	$upsOutputSource =  getsnmpOID($snmp,$upsOutputSource);
	if (($upsOutputSource !~ /^noSuch/) && ($upsBatteryStatus !~ /^noSuch/)) 
	{
        if($Sysoid_value==232)
        {
            my $oid_hp_input_source = "1.3.6.1.4.1.232.165.3.3.5.0";
            my %oid_hp_input_sourceMessage = (
		            "1" => "other(1)",
		            "2" => "none(2)",
		            "3" => "primaryUtility(3)",
		            "4" => "bypassFeed(4)",
		            "5" => "secondaryUtility(5)",
		            "6" => "generator(6)",
		            "7" => "flywheel(7)",
		            "8" => "fuelcell(8)"
	            );
            my %oid_hp_input_sourceStatus = (
		            "1" => "UNKNOWN",
		            "2" => "OK",
		            "3" => "OK",
		            "4" => "CRITICAL",
		            "5" => "WARNING",
		            "6" => "WARNING",
		            "7" => "WARNING",
		            "8" => "WARNING"
	            );
            my $hpresult = $snmp->get_request("$oid_hp_input_source");
            my $inputstate = $$hpresult{"$oid_hp_input_source"};
            $output = "UPS State is ".$oid_hp_input_sourceMessage{$inputstate}.". Battery State is ".$upsBatteryStatuses{$upsBatteryStatus}.". ";
            $state = $oid_hp_input_sourceStatus{$inputstate};
            $return = priority_state($return,$ERRORS{$state});
        }
        elsif($upsOutputSource == 2 && $Sysoid_value==318)
        {
            my $oid_battery_capacity = ".1.3.6.1.4.1.318.1.1.1.2.2.1.0";
            my $oid_battery_temperature = ".1.3.6.1.4.1.318.1.1.1.2.2.2.0";
            my $oid_battery_replace = ".1.3.6.1.4.1.318.1.1.1.2.2.4.0";
            my $apcresult = $snmp->get_request("$oid_battery_capacity");
            my $apccapacity = $$apcresult{"$oid_battery_capacity"};
            $apcresult = $snmp->get_request("$oid_battery_replace");
            my $apcreplace = $$apcresult{"$oid_battery_replace"};
            $apcresult = $snmp->get_request("$oid_battery_temperature");
            my $apctemperature = $$apcresult{"$oid_battery_temperature"};    
            my $apcmessage="";
            if($apcreplace !=1)
            {
                $state='WARNING';
                $return = priority_state($return,$ERRORS{$state});
                $apcmessage='Battery must be change'; 
            }
            else
            {
                $upsOutputSource=3;
                $apcmessage="UPS state is OK (temperature: $apctemperature Celsius)."; 
            }
            $output = "$apcmessage\nBattery load: $apccapacity%\nBattery State is ".$upsBatteryStatuses{$upsBatteryStatus}."\n";
		    if ($upsOutputSource == 3) 
		    { # normal
			    $state = 'OK';
		    } 
		    elsif (($upsOutputSource == 6) || ($upsOutputSource == 7)) 
		    { #6 ==booster 7 ==reducer
			    $state = 'WARNING';
                $return = priority_state($return,$ERRORS{$state});
		    } 
		    else 
		    {
			    $state = 'CRITICAL';
                $return = priority_state($return,$ERRORS{$state});
		    }

		    if (($state eq 'OK') && ($upsBatteryStatus != 2)) 
		    { # battery low or depleted
			    $state = 'WARNING';
                $return = priority_state($return,$ERRORS{$state});
		    }
        }
        else
        {
            $output = "UPS State is ".$upsOutputSourcesMessage{$upsOutputSource}.". Battery State is ".$upsBatteryStatuses{$upsBatteryStatus}.". ";
		    if ($upsOutputSource == 3) 
		    { # normal
			    $state = 'OK';
		    } 
		    elsif (($upsOutputSource == 6) || ($upsOutputSource == 7)) 
		    { #6 ==booster 7 ==reducer
			    $state = 'WARNING';
                $return = priority_state($return,$ERRORS{$state});
		    } 
		    else 
		    {
			    $state = 'CRITICAL';
                $return = priority_state($return,$ERRORS{$state});
		    }

		    if (($state eq 'OK') && ($upsBatteryStatus != 2)) 
		    { # battery low or depleted
			    $state = 'WARNING';
                $return = priority_state($return,$ERRORS{$state});
		    }
        }
		$output = $output."Time remaining is ".$upsEstimatedMinutesRemaining." min\n";
        print "$state : $output";
		my $indexLine = $snmp->get_table(Baseoid => $upsOutputLineIndex);
		my @array;
		if (!defined($indexLine)) 
		{
			for(my $i=1;$i<10;$i++)
			{
				my $value="$i";
				push @array,$value;
			}
		}
		else
		{
			foreach my $key ( keys %$indexLine) 
			{
				push @array,$$indexLine{$key};
			}
		}
		foreach $indexLine (@array)
		{
			my @upsoid;
			push (@upsoid,$upsOutputVoltage.$indexLine);
			push (@upsoid,$upsOutputCurrent.$indexLine);
			push (@upsoid,$upsOutputPower.$indexLine);
			push (@upsoid,$upsOutputPercentLoad.$indexLine);
			$result = $snmp->get_request( -varbindlist => \@upsoid);
			#corrige un bug avec certain UPS (riello) qui renvoit la première ligne en boucle.
			if (!defined($result)) 
			{
				last;
			}
			elsif(!defined($result->{$upsOutputVoltage.$indexLine}))
			{
				last;
			}
			my $linestatus = "OK";
			my $loadmessage="";
			if($result->{$upsOutputPercentLoad.$indexLine} > $lineLoadCritical)
			{
				$loadmessage=$result->{$upsOutputPercentLoad.$indexLine}."% load > $lineLoadCritical";
				$linestatus='CRITICAL';
                $return = priority_state($return,$ERRORS{$linestatus});
			}
			elsif($result->{$upsOutputPercentLoad.$indexLine} > $lineLoadWarning)
			{
				$loadmessage=$result->{$upsOutputPercentLoad.$indexLine}."% load > $lineLoadWarning";
				$linestatus='WARNING';
                $return = priority_state($return,$ERRORS{$linestatus});
			}
			else
			{
				$loadmessage=$result->{$upsOutputPercentLoad.$indexLine}."% load < $lineLoadWarning";
			}
			print "$linestatus:  Line $indexLine: ".$result->{$upsOutputVoltage.$indexLine}." Volt, ".$result->{$upsOutputPower.$indexLine}." Watt, ".$loadmessage.".\n";
		}
        $result = $snmp->get_table(Baseoid => $oid_upsAlarm);
        foreach my $key ( keys %$result) 
        {	
            if($key=~/3\.6\.1\.2\.1\.33\.1\.6\.2\.1\.2/)
            {
                my $descr =$$result{"$key"};
                if($descr=~/1.3.6.1.2.1.33.1.6/)
                {
                    my $state = $upsOutputAlarmStatus{$descr};
                    print "$state : alarm message: ".$upsOutputAlarmMessage{$descr}."\n";
                    $return = priority_state($return,$ERRORS{$state});
                }
            }
        }
	}
}
exit $return;

sub print_help() {
	print_revision($PROGNAME,'$Revision: 0.1 $ ');
	print "Copyright (c) 2019 Esia SA\n";
	print "\n";
	printf "Check UPS status via SNMP.\n";
	print "\n";
	print_usage();
	print "\n";
	print " -H (--hostname)     Hostname to query - (required)\n";
	print " -C (--community)    SNMP read community or v3 auth (defaults to public)\n";
	print "                     (v3 specified as username:authpassword:... )\n";
	print "                       username = SNMPv3 security name\n";
	print "                       authpassword = SNMPv3 authentication pass phrase (or hexidecimal key)\n";
	print "                       authprotocol = SNMPv3 authentication protocol (md5 (default) or sha)\n";
	print "                       privpassword = SNMPv3 privacy pass phrase (or hexidecmal key)\n";
	print "                       privprotocol = SNMPv3 privacy protocol (des (default) or aes)\n";
	print " -v (--snmp_version) 1 for SNMP v1 default)\n";
	print "                     2 for SNMP v2c\n";
	print " -w (--warning)  value of warning load. Default: 70 percent\n";
	print " -c (--critical) value of critical load. Default: 80 percent\n";
	print " -V (--version)      Plugin version\n";
	print " -h (--help)         usage help\n";
	print "\n";
	support();
}

sub print_usage() {
	print "Usage: \n";
	print "  $PROGNAME -H <HOSTNAME> [-C <community>]\n";
	print "  $PROGNAME [-h | --help]\n";
	print "  $PROGNAME [-V | --version]\n";
}

