#!/usr/bin/perl -w
############################## check_wmi_security_center #################
my $Version='1.0';
# Date : Mar 28 2017
# Author  : Nathan Faes
# Help : esia-sa.com
############################################################################

use strict;
use warnings;
use Getopt::Long;
use Text::Trim;

use lib '/usr/local/gesa/plugins/lib';

my $PROGNAME='check_wmi_security_center.pl';
my $delimiter="§";
my ($opt_h,$opt_V);
my $hostname;
my $spyware;
my $antivirus;
my $firewall;
my $return=0;
 

my %columns=(
	"spyware"	=> "displayName,productState",
	"antivirus"	=> "displayName,productState",
	"firewall"	=> "displayName,productState",
);

my %tables=(
    "spyware"   => "AntiSpywareProduct",
	"antivirus"	=> "AntiVirusProduct",
	"firewall"	=> "FirewallProduct",
);

my %namespaces=(
    "spyware"   => 'root\securitycenter2',
	"antivirus"	=> 'root\securitycenter2',
    "firewall"  => 'root\securitycenter2',
);

my %ERRORS=( 
        "OK"            => 0,
        "WARNING"       => 1,
        "CRITICAL"      => 2,
        "UNKNOWN"       => 3,
);

my @priorities=(
                $ERRORS{CRITICAL},
                $ERRORS{WARNING},
                $ERRORS{UNKNOWN},
                $ERRORS{OK});
sub priority_state{
        my($a,$b) = @_;
        if( $a == $b ) { return $a; } #pas besoin d'aller dans la boucles si ils sont identiques

        foreach my $v (@priorities){
                if ( $v == $a || $v == $b ) { return $v; }
        }
}

my %states;
$states{262144}{'def'}="Updated";
$states{262144}{'protection'}="Disabled";
$states{262160}{'def'}="Outdated";
$states{262160}{'protection'}="Disabled";
$states{266240}{'def'}="Updated";
$states{266240}{'protection'}="Enabled";
$states{266256}{'def'}="Outdated";
$states{266256}{'protection'}="Enabled";
$states{393216}{'def'}="Updated";
$states{393216}{'protection'}="Disabled";
$states{393232}{'def'}="Outdated";
$states{393232}{'protection'}="Disabled";
$states{393488}{'def'}="Outdated";
$states{393488}{'protection'}="Disabled";
$states{397312}{'def'}="Updated";
$states{397312}{'protection'}="Enabled";
$states{397328}{'def'}="Outdated";
$states{397328}{'protection'}="Enabled";
$states{397584}{'def'}="Outdated";
$states{397584}{'protection'}="Enabled";
$states{393472}{'def'}="Updated";
$states{393472}{'protection'}="Disabled";
$states{397568}{'def'}="Updated";
$states{397568}{'protection'}="Enabled";
my $auhfile = "";


sub error_exit_unknown{
	my ($line) = @_;
	if($line =~ m/(ERROR:.*)/)	{
		print $1 . "\n";
		exit $ERRORS{UNKNOWN};
	}
}

sub print_help() {
	printf "Check Windows Security Center.\n";
	print "\n";
	print_usage();
	print "\n";
	print " -H (--hostname)     Hostname to query - (required)\n";
	print " -A (--authfile)     Authentification file\n";
	print " -s (--spyware)      AntiSpyware informations\n";
	print " -v (--antivirus)    AntiVirus informations\n";
	print " -f (--firewall)		Firewall informations\n";
	print " -V (--version)      Plugin version\n";
	print " -h (--help)         usage help\n";
	print "\n";
}

sub print_usage() {
	print "Usage: \n";
	print "  $PROGNAME -H <HOSTNAME> [-s] [-v] [-f]\n";
	print "  $PROGNAME [-h | --help]\n";
	print "  $PROGNAME [-V | --version]\n";
}

&Getopt::Long::config('bundling');
GetOptions(
	"V"   	=> \$opt_V,"version"		=> \$opt_V,
	"h"   	=> \$opt_h,"help"			=> \$opt_h,
	"H=s" 	=> \$hostname,"hostname=s" 	=> \$hostname,
	"s" 	=> \$spyware,"spyware" 		=> \$spyware,
	"v" 	=> \$antivirus,"antivirus" 	=> \$antivirus,
	"f" 	=> \$firewall,"firewall" 	=> \$firewall,
	"A=s" 	=> \$auhfile,"authfile=s" 	=> \$auhfile,
);

# -h & --help print help
if ($opt_h) 
{ 
	print_help(); 
	exit $ERRORS{'OK'}; 
}
if (!$hostname) 
{ 
	print_help(); 
	exit $ERRORS{'UNKNOWN'}; 
}
my $wmic_command="wmic -A $auhfile --delimiter=$delimiter";
sub prepare_request{
	my ($mode) = @_;

	my $table=$tables{$mode};
	my $column=$columns{$mode};

	my $req="SELECT $column FROM $table";
	
	return $req;	
}

sub send_request{
	my ( $ip, $req, $mode) = @_;
	my @output;
	my $namespace=$namespaces{$mode};
	
	error_exit_unknown("ERROR: No Host defined")		unless defined $ip;
	error_exit_unknown("ERROR: No request defined")		unless defined $req;
 
	my $command=$wmic_command;
	$command = $command. " --namespace='" . $namespace . "' " if defined $namespace;
	$command=$command . " //" . $ip  . " '" . $req . "' 2>&1";

	my $i=0;

	my $idx="nl";
    my $firstLine="<EMPTY>";
	my $ret="";

	open(my $results,$command . "|");
	while( my $line = <$results> )
	{
        #error_exit_unknown($line);
		#return @output if ( $i > $max ); 
	
		if( $line !~ m/CLASS:/ && $firstLine ne $line )
		{
            $firstLine=$line if $i==0;

			$idx=$i	if $i>0;
            $line="$idx$delimiter$line";

			chomp $line;
			#$ret=$protocol->addLine(selectColumns($cl,split(/$delimiter/,$line)))	if $i >= $min ;
			#$ret and last;
			$ret = $ret.$line."\n";
			$i++;
		}	
	}
    
	close $results;
	return $ret;
}
my $request;
my $out;
my $all=0;

if(!$spyware && !$antivirus && !$firewall)
{
	$all=1;
}

if($spyware || $all==1)
{
	$request = prepare_request("spyware");
	$out = send_request($hostname,$request,"spyware");
	my @spy = split(/\n/,$out);
	my $j=0;
	my @elements;
	my @enabled;
	if(@spy)
	{
		print "Spyware : ";
		foreach my $element (@spy)
		{
			#print "$element\n";
			if($j>0)
			{
				my @name = split(/§/,$element);
				push @elements, $name[1]." : ".$states{$name[3]}{"def"}." and ".$states{$name[3]}{"protection"};
			
				if($states{$name[3]}{"protection"} eq 'Enabled')
				{
					push @enabled, $name[1]." : ".$states{$name[3]}{"def"}." and ".$states{$name[3]}{"protection"};
				}
			}
			$j++;
		}

		#Si il y a des activés, je n'affiche que ceux là et ne vérifie que l'état de ceux là
		if(@enabled)
		{
			foreach my $ena (@enabled)
			{
				print $ena." ; ";
				my @update = split(/:/,$ena);
				#On renvoie une erreur critique si l'élément n'est pas à jour
				if($update[1] =~ m/Outdated/)
				{
					$return = priority_state($return,$ERRORS{CRITICAL});
				}			
			}
		}
		#Sinon on affiche tout et on renvoie une erreur critique quoiqu'il arrive puisque aucun n'est activé
		else
		{
			foreach my $ele (@elements)
			{
				print $ele." ; ";	
			}
			$return = priority_state($return,$ERRORS{CRITICAL});		
		}
		print "\n";
	}
}

if($antivirus || $all==1)
{
	$request = prepare_request("antivirus");
	$out = send_request($hostname,$request,"antivirus");
	my @spy = split(/\n/,$out);
	my $j=0;
	my @elements;
	my @enabled;
	if(@spy)
	{
		print "Antivirus : ";
		foreach my $element (@spy)
		{
			#print "$element\n";
			if($j>0)
			{
				my @name = split(/§/,$element);
				push @elements, $name[1]." : ".$states{$name[3]}{"def"}." and ".$states{$name[3]}{"protection"};
			
				if($states{$name[3]}{"protection"} eq 'Enabled')
				{
					push @enabled, $name[1]." : ".$states{$name[3]}{"def"}." and ".$states{$name[3]}{"protection"};
				}
			}
			$j++;
		}

		#Si il y a des activés, je n'affiche que ceux là et ne vérifie que l'état de ceux là
		if(@enabled)
		{
			foreach my $ena (@enabled)
			{
				print $ena." ; ";
				my @update = split(/:/,$ena);
				#On renvoie une erreur critique si l'élément n'est pas à jour
				if($update[1] =~ m/Outdated/)
				{
					$return = priority_state($return,$ERRORS{CRITICAL});
				}			
			}
		}
		#Sinon on affiche tout et on renvoie une erreur critique quoiqu'il arrive puisque aucun n'est activé
		else
		{
			foreach my $ele (@elements)
			{
				print $ele." ; ";	
			}
			$return = priority_state($return,$ERRORS{CRITICAL});		
		}
		print "\n";
	}
	else
	{
		print "CRITICAL: no antivirus found";
		$return = priority_state($return,$ERRORS{CRITICAL});
	}
}

if($firewall || $all==1)
{
	$request = prepare_request("firewall");
	$out = send_request($hostname,$request,"firewall");
	my @spy = split(/\n/,$out);
	my $j=0;
	my @elements;
	my @enabled;
	if(@spy)
	{
		print "Firewall : ";
		foreach my $element (@spy)
		{
			#print "$element\n";
			if($j>0)
			{
				my @name = split(/§/,$element);
				push @elements, $name[1]." : ".$states{$name[3]}{"def"}." and ".$states{$name[3]}{"protection"};
			
				if($states{$name[3]}{"protection"} eq 'Enabled')
				{
					push @enabled, $name[1]." : ".$states{$name[3]}{"def"}." and ".$states{$name[3]}{"protection"};
				}
			}
			$j++;
		}

		#Si il y a des activés, je n'affiche que ceux là et ne vérifie que l'état de ceux là
		if(@enabled)
		{
			foreach my $ena (@enabled)
			{
				print $ena." ; ";
				my @update = split(/:/,$ena);
				#On renvoie une erreur critique si l'élément n'est pas à jour
				if($update[1] =~ m/Outdated/)
				{
					$return = priority_state($return,$ERRORS{CRITICAL});
				}			
			}
		}
		#Sinon on affiche tout et on renvoie une erreur critique quoiqu'il arrive puisque aucun n'est activé
		else
		{
			foreach my $ele (@elements)
			{
				print $ele." ; ";	
			}
			$return = priority_state($return,$ERRORS{CRITICAL});		
		}
		print "\n";
	}
}

exit($return);