#!/usr/bin/perl -w
############################## check_wmi_user_connect #################
my $Version='1.0';
# Date : Mar 28 2017
# Author  : Nathan Faes
# Help : esia-sa.com
############################################################################

use strict;
use warnings;
use Getopt::Long;
use Text::Trim;
use POSIX qw(strftime);
use lib '/usr/local/gesa/plugins/lib';

my $PROGNAME='check_wmi_user_connect.pl';
my $delimiter="§";
my ($opt_h,$opt_V);
my $hostname;
my $authfile ='/etc/esia/wmi.conf';
my $username;

sub print_help() {
	printf "Check Windows user connection.\n";
	print "\n";
	print_usage();
	print "\n";
	print " -H (--hostname)     Hostname to query - (required)\n";
	print " -A (--authfile)     Authentification file\n";
	print " -u (--user)      	the username\n";
	print " -V (--version)      Plugin version\n";
	print " -h (--help)         usage help\n";
	print "\n";
}

sub print_usage() {
	print "Usage: \n";
	print "  $PROGNAME -H <HOSTNAME> [-u] [-v] [-f]\n";
	print "  $PROGNAME [-h | --help]\n";
	print "  $PROGNAME [-V | --version]\n";
}
my %ERRORS=( 
        "OK"            => 0,
        "WARNING"       => 1,
        "CRITICAL"      => 2,
        "UNKNOWN"       => 3,
);

my @priorities=(
                $ERRORS{CRITICAL},
                $ERRORS{WARNING},
                $ERRORS{UNKNOWN},
                $ERRORS{OK});
sub priority_state{
        my($a,$b) = @_;
        if( $a == $b ) { return $a; } #pas besoin d'aller dans la boucles si ils sont identiques

        foreach my $v (@priorities){
                if ( $v == $a || $v == $b ) { return $v; }
        }
}

&Getopt::Long::config('bundling');
GetOptions(
	"V"   	=> \$opt_V,"version"		=> \$opt_V,
	"h"   	=> \$opt_h,"help"			=> \$opt_h,
	"H=s" 	=> \$hostname,"hostname=s" 	=> \$hostname,
	"u=s" 	=> \$username,"username=s" 	=> \$username,
	"A=s" 	=> \$authfile,"authfile=s" 	=> \$authfile,
);

# -h & --help print help
if ($opt_h) 
{ 
	print_help(); 
	exit $ERRORS{'OK'}; 
}
if (!$hostname) 
{ 
	print_help(); 
	exit $ERRORS{'UNKNOWN'}; 
}
my $timestamp = time();
$timestamp=$timestamp-16000;
my $datestring = strftime("%Y%m%d%H%M%S", localtime($timestamp));
$datestring=$datestring.'.00000000';

#my $wmic_command="wmic -A $authfile --delimiter=$delimiter //$hostname 'Select ComputerName,EventCode,TimeGenerated,Type,message FROM Win32_NTLogEvent";
#$wmic_command .=" WHERE (EventCode=4624 OR EventCode=4634 OR EventCode=4647 OR EventCode=4778 OR EventCode=4779 OR EventCode=4801 OR EventCode=4800) AND Message LIKE \"%".$username."%\" AND TimeGenerated > \"$datestring\"'";

my $wmic_command="wmic -A $authfile --delimiter=$delimiter //$hostname 'SELECT * FROM Win32_LoggedOnUser'";
#print "$wmic_command\n";
#print "$wmic_command\n";
open(my $results,$wmic_command . "|");
my $firstLine="<EMPTY>";
my $status='OK';
my @event;
my $return= $ERRORS{OK};
my @loginid;
#512 / 4608  STARTUP
#513 / 4609  SHUTDOWN
#528 / 4624  LOGON
#538 / 4634  LOGOFF
#551 / 4647  BEGIN_LOGOFF
#N/A / 4778  SESSION_RECONNECTED
#N/A / 4779  SESSION_DISCONNECTED
#N/A / 4800  WORKSTATION_LOCKED
#* / 4801    WORKSTATION_UNLOCKED
#N/A / 4802  SCREENSAVER_INVOKED
#N/A / 4803  SCREENSAVER_DISMISSED
my $found=0;
while( my $line = <$results> )
{
	if( $line !~ m/CLASS:/ && $line !~ m/Antecedent/ )
	{
		chomp $line;
		if($line =~ /failed NT status/)
		{
			$return= $ERRORS{UNKNOWN};
			close $results;
			print "UNKNOWN: WMI error, check the host or the parameter\n";
			exit($return);
		}
		else
		{
			@event = split(/§/,$line);
			if($event[0]=~ /$username/)
			{
				if($event[1]=~ /\.LogonId=\"([0-9]*)/)
				{
					push(@loginid,$1);
					$found=1;
				}
			}
		}
	}
}
close $results;

#Si on a un utilisateur connecté on regarde s'il est connecté ou déconnecté
my $value=0;
if($found==1)
{
	#on récupère l'ensemble des sessions qui sont en RDP
	$wmic_command="wmic -A $authfile --delimiter=$delimiter //$hostname 'SELECT * FROM Win32_LogonSession WHERE LogonType=10' ";

	my $secondwmic ="wmic -A $authfile --delimiter=$delimiter //$hostname 'SELECT * FROM Win32_SessionProcess'";
	#print "$wmic_command\n";
	#print "$secondwmic\n";

	open(my $results2,$wmic_command . "|");
	while( my $line2 = <$results2> )
	{
		if( $line2 !~ m/CLASS:/ && $line2 !~ m/AuthenticationPackage/ )
		{
			my @logon = split(/§/,$line2);
			#On a bien un session en remote RDP
			if ( grep( /^$logon[4]$/, @loginid ) ) 
			{			
				open(my $results3,$secondwmic . "|");
				my $message = "disconnect";
				my $state = "WARNING";
				$return = priority_state($return,$ERRORS{WARNING});
				#my $wmi_commandExec = "wmic -A $authfile --delimiter=$delimiter //$hostname 'SELECT Name,HandleCount,UserModeTime,PageFileUsage,priority,ThreadCount,SessionId FROM Win32_Process WHERE (Name=\"explorer.exe\" OR Name=\"LogonUI.exe\" )";
				my $wmi_commandExec = "wmic -A $authfile --delimiter=$delimiter //$hostname 'SELECT Name,ThreadCount FROM Win32_Process WHERE (Name=\"explorer.exe\" OR Name=\"LogonUI.exe\" )";
				my $flagfirst =0;
					#open(my $results3,wmi_commandExec . "|");
				#on execute la requete session process afin de connaitre les processus exécutés par les utilisateurs et construire la dernière requête.
				while( my $line = <$results3> )
				{
					if($line=~ /\.LogonId=\"$logon[4]/)
					{
						if($line=~ /\.Handle=\"([0-9]*)/)
						{
							my $processid = $1;
							if($flagfirst==0)
							{
								$wmi_commandExec .=" AND ( ProcessId=$processid";
								$flagfirst=1;
							}
							else
							{
								$wmi_commandExec .=" OR ProcessId=$processid";
							}
						}
					}
				}
				close $results3;
				if ($flagfirst==1)
				{
					$wmi_commandExec .=")'";
				}
				if($flagfirst==1)
				{
					open($results3,$wmi_commandExec . "|");
					my $count_explorer = 0;
					my $count_LogonUi = 0;
					my $flagdisconnect = 0;
					while( my $line = <$results3> )
					{
						if( $line !~ m/CLASS:/ )#&& $line !~ m/Handle/ )
						{
							#my $timestamp = localtime(time);
							my @process = split(/§/,$line);
							if($process[1]=~ /LogonUI\.exe/)
							{	
								$count_LogonUi++;
							}
							elsif($process[1]=~ /explorer\.exe/)
							{
								$count_explorer++;
								if($process[2] < 42)
								{
									$flagdisconnect=1;
								}
							}
						}
					}	
					close $results3;			
					$value++;
					if($count_explorer == 1 && $count_LogonUi > 0)
					{
						$message = "locked";
						$state = "WARNING";
						$return = priority_state($return,$ERRORS{WARNING});
					}
					elsif($count_explorer == 0)
					{
						$message = " not logged";
						$state = "OK";
						$return = priority_state($return,$ERRORS{OK});
					}
					else
					{
						if($flagdisconnect ==1)
						{
							$message = "disconnect or not active";
							$state = "WARNING";
							$return = priority_state($return,$ERRORS{WARNING});
						}
						else
						{
							$message = "login";
							$state = "CRITICAL";
							$return = priority_state($return,$ERRORS{CRITICAL});
						}
					}
					print "$state: $username is $message";
				}
			}
		}
	}
	close $results2;
}
else
{
	$return = $ERRORS{OK};
	print "OK: $username is not logged on $hostname"; 
}
print "|data=$value\n";
exit($return);
