#!/usr/bin/perl
use strict;
use warnings;
use Getopt::Std;
use Text::Trim;


my $wmic_command="wmic -A  /etc/esia/wmi.conf";
my %ret_codes=(   #TODO : renomer en ERRORS
        "OK"            => 0,
        "WARNING"       => 1,
        "CRITICAL"      => 2,
        "UNKNOWN"       => 3,
);

my %reverse_ret_codes=reverse %ret_codes;

my %request=(
	"services"	=> '"SELECT State,StartMode FROM Win32_Service"',
	"ram_available"	=> '"SELECT AvailableBytes FROM Win32_PerfRawData_PerfOS_Memory"',
#	"ram_capacity"	=> '"SELECT Capacity FROM Win32_PhysicalMemory"',
	"ram_capacity" 	=> '"SELECT TotalPhysicalMemory FROM Win32_ComputerSystem"',
	"part"		=> '"SELECT FreeSpace, Size FROM Win32_LogicalDisk"',
	"cpu"		=> '"SELECT LoadPercentage FROM Win32_Processor"',
	"iodisk"	=> '"SELECT Name,DiskReadBytesPersec,DiskWritesPersec FROM Win32_PerfFormattedData_PerfDisk_PhysicalDisk"',
	"smart"		=> '"SELECT Caption,Status from Win32_DiskDrive"',
);

my %calc_bytes=(

	"KB"	=> 1024,
	"MB"	=> 1048576,
	"GB"	=> 1073741824,
	"TB"	=> 1024**4,
);

my %opts;

sub help{
	print "Use : \n";
	print "\t-m services -n <list> -s <word> -S <word>\n";#s=state S=Started
	print "\n";
	print "Options : \n";
	print "\t-m <word>\tDefine the check Mode (services)\n";
	print "\t-n <list>\tDefine the item checked\n";
}


my @priorities=(
                $ret_codes{CRITICAL},
                $ret_codes{WARNING},
                $ret_codes{UNKNOWN},
                $ret_codes{OK}
);

sub priority_state{
	my($a,$b) = @_;

	if( $a == $b ) { return $a; } #pas besoin d'aller dans la boucles si ils sont identiques

	foreach my $v (@priorities){
		if ( $v == $a || $v == $b ) { return $v; }
	}
}

sub error_exit_unknown{
	my ($line) = @_;
	if($line =~ m/(ERROR:.*)/)	{
		print $1 . "\n";
		exit $ret_codes{UNKNOWN};
	}
}

sub filter{
	my ($line,$regs) = @_;
	if(defined $regs){
		foreach my $v (split(/,/,$regs)){
			if ($line =~ m/$v/) {return 1;}
		}
		return 0;
	}
	return 1;
}

sub services_mode{
	my $command=$wmic_command . ' //' . $opts{H} . " " . $request{services} . ' 2>&1';
	my @output;
	my $i=0;
	print $command . "\n";

	my 			$error_state=$ret_codes{CRITICAL};
	if(defined $opts{w}) {	$error_state=$ret_codes{WARNING};    }
	if(defined $opts{c}) {  $error_state=$ret_codes{CRITICAL};   } #Ecrase le warning si -c setted
	
	my $total_state;


	my $arg_state="EMPTY";
	my $arg_startMode="EMPTY";

	if(defined $opts{S}){
		$arg_state=lc(trim($opts{S}));
		if	( $arg_state eq "running" ) { $arg_state="Running"; }
		elsif	( $arg_state eq "stopped" ) { $arg_state="Stopped"; }
		else{
			print "ERROR: State parameter must be Running or Stopped\n";
			help();
	                exit $ret_codes{UNKNOWN};
		}
	}

	if(defined $opts{s}){
                $arg_startMode=lc(trim($opts{s}));
                if      ( $arg_startMode eq "auto" ) { $arg_startMode="Auto"; }
                elsif   ( $arg_startMode eq "manual" ) { $arg_startMode="Manual"; }
                else{
                        print "ERROR: StartMode parameter must be Auto or Manual\n";
                        help();
                        exit $ret_codes{UNKNOWN};
                }
        }

	open(my $results,$command . "|");
	while(my $line= <$results>){#bcl for read response, check errors and filter
		error_exit_unknown($line);#check for errors, print to display not possible in this bcl
		if($i>2&& !( $line =~ m/CLASS:/)){
			my ($name,$startMode,$state) = split(/\|/,$line,3);
			chomp $state;

			if(filter($name,$opts{n})){
				push @output,$line;
			}
		}
		$i++;
	}
	close $results;

	$total_state=-1;
	foreach my $item (@output){	
		my ($name,$startMode,$state) = split(/\|/,$item,3);	
		chomp $state;

		my $actual_state=$ret_codes{OK};
		if($arg_state ne "EMPTY" && $arg_state ne $state ) 		{  $actual_state=$error_state; }
		if($arg_startMode ne "EMPTY" && $arg_startMode ne $startMode ) 	{  $actual_state=$error_state; }
		print $reverse_ret_codes{$actual_state} . " - " .$name . " is " . $state . " ( " . $startMode . " )\n";

		$total_state=priority_state($total_state,$actual_state);
	}
	if( $total_state == -1 ) { exit $ret_codes{UNKNOWN}; }
	exit $total_state;
}

sub read_strings{
	my ( $req, $regex, $lim ) = @_;
	my @output;
        my $command=$wmic_command . ' //' . $opts{H} . " " . $req . ' 2>&1';
	my $i=0;
	
	$lim=1 unless defined $lim;

	open(my $results,$command . "|");
        while(my $line= <$results>){
                error_exit_unknown($line);
                if($i>$lim && !( $line =~ m/CLASS:/)){
                        my ($name,$data ) = split(/\|/,$line,2);
                        chomp $data;

                        if(filter($name,$regex)){
                                push @output,$line;
                        }
                }
                $i++;
        }
        close $results;

	return @output;
}

sub read_int{
	my ( $req ) = @_;
	my $line;
	my $old;
 	my $command=$wmic_command . ' //' . $opts{H} . " " . $req . ' 2>&1';

        open(my $results,$command . "|cut -d'|' -f1 | ");
        while( $line = <$results>){#the last line is the response
                error_exit_unknown($line);#check for errors, print to display not possible in this bcl
		$old=$line;
        }
        close $results;

        $old=trim($old);
        if ($old !~  m/^\d+$/) { error_exit_unknown("ERROR: The output isn't one integer"); }
	return int($old);
}


sub parse_usage{
	my ( $a, $c,$p, $unit) = @_;

	my $available= int($a/$calc_bytes{$unit});
        my $capacity= int($c/$calc_bytes{$unit});
	my $usage=int($capacity-$available);

	return "Usage : $usage $unit ($p%) | Free : $available/$capacity $unit\n" ;
}

sub configure_unit{
	my ( $unit ) = @_; #param par défaut
	if(defined $opts{u}){
		my $u=lc(trim($opts{u}));
                if      ( $u eq "k" ) { $unit="KB"; }
                elsif   ( $u eq "m" ) { $unit="MB"; }
                elsif   ( $u eq "g" ) { $unit="GB"; }
		elsif 	( $u eq "t" ) { $unit="TB"; }
        }
	return $unit;
}

#sub check_alert_state{
#	my ( $usage_percents ) = @_;

   	#Configuration des paramètres warning et critical
#        my $warn=80;
#        my $crit=90;
#        if(defined $opts{w}){
#                $warn=trim($opts{w});
#                if($warn !~ m/^\d+$/) { error_exit_unknown("ERROR: The -w param must only take one integer"); }
#                if($crit<$warn) { $crit=$warn; }
#        }
#        if(defined $opts{c}){
#                $crit=trim($opts{c});
#                if($crit !~ m/^\d+$/) { error_exit_unknown("ERROR: The -c param must only take one integer"); }
#                unless ( defined $opts{w}) { $warn=$crit; }
#        }

#        if( $crit < $warn ) { error_exit_unknown("ERROR: The -c param must be bigger or equal than -w parameter"); }

        #Vérification du status OK, WARNING or CRITICAL
#        my $state=$ret_codes{OK};
#        if ( $warn <= $usage_percents ) { 
#                if( $crit <= $usage_percents )  { $state=$ret_codes{CRITICAL}; }
#                else                            { $state=$ret_codes{WARNING}; }
#        }

#	return $state;
#}

sub parse_alert{
	my ( $name, $alert ) = @_;
	$alert=trim($alert);
        if($alert !~ m/^\d+$/) { error_exit_unknown("ERROR: The $name param must only take one integer"); }
	return $alert;
}

sub single_alert_state{
        my ( $val, $warn, $crit ) = @_;

#	print "ALERT_STATE $val $warn $crit \n";

        if( $crit < $warn ) { error_exit_unknown("ERROR: The -c parameter must be bigger or equal than -w parameter"); }

        #Vérification du status OK, WARNING or CRITICAL
        my $state=$ret_codes{OK};
        if ( $warn <= $val ) {
                if( $crit <= $val )  { $state=$ret_codes{CRITICAL}; }
                else                 { $state=$ret_codes{WARNING}; }
        }

        return $state;
}

sub check_alert_state{
	my ( $vals,$wds, $cds  ) = @_;
	
	my ( $val1, $val2 )	= split(/,/,$vals);
	my ( $warn1, $warn2 )   = split(/,/,$wds);
	my ( $crit1, $crit2 )   = split(/,/,$cds);

	if(defined $opts{w}){ 
		( $warn1, $warn2 )   = split(/,/,$opts{w});
		$warn1=parse_alert("-w X",$warn1);
		if ( $warn1 > $crit1 ) { $crit1 = $warn1; }
	}

	if(defined $opts{c}) 	{ 
		( $crit1, $crit2 )   = split(/,/,$opts{c}); 
		$crit1=parse_alert("-c X",$crit1);	
		unless ( defined $opts{w} || $warn1 < $crit1 )  { $crit1 = $warn1; }
	}

	my $state=single_alert_state($val1,$warn1,$crit1);
	if(defined $val2){
		unless (defined $warn2) { $warn2=$warn1; }
		else 			{ $warn2=parse_alert("-w $warn1,X",$warn2); }
		unless (defined $crit2) { $crit2=$crit1; }
		else 			{ $crit2=parse_alert("-c $crit1,X",$crit2); }

		return priority_state($state,single_alert_state($val2,$warn2,$crit2));
	}	
	return $state;
}

sub ram_usage{
        my $unit=configure_unit("MB");

        #Récupération des données
        my $available_bytes=read_int($request{ram_available});
  #      my $capacity_bytes=read_int($request{ram_capacity});
	my @output=read_strings($request{ram_capacity});
	my ( $name,$capacity_bytes ) =split(/\|/,$output[0]);
        my $usage_bytes=$capacity_bytes-$available_bytes;

        my ( $usage_percents,$state ) ;
        if ( $capacity_bytes != 0 ) {
                $usage_percents=int($usage_bytes/($capacity_bytes/100));
                $state=check_alert_state($usage_percents,80,90);
        }
        else {
                $usage_percents="unknown ";
                $state=$ret_codes{UNKNOWN};
        }

        #Envoi des informations (STDOUT et valeur de retour)
        print "$reverse_ret_codes{$state} - ". parse_usage($available_bytes,$capacity_bytes,$usage_percents,$unit);
        exit $state;
}


sub part_usage{
	my $unit=configure_unit("GB");
	my $total_state=-1;
	
	foreach my $v ( read_strings($request{part},$opts{n})){
		#data parsing
		unless ( $v =~ m/(.*)\|(\d*)\|(\d*)/ ) { error_exit_unknown("ERROR: Error to parse receiving data"); }
		my $letter=$1;
		my $available_bytes=$2;
		my $capacity_bytes=$3;
		my $usage_bytes=$capacity_bytes-$available_bytes;

  		my ( $usage_percents,$state ) ;
        	if ( $capacity_bytes != 0 ) {
                	$usage_percents=int($usage_bytes/($capacity_bytes/100));
                	$state=check_alert_state($usage_percents,80,90);
       		}
        	else {
                	$usage_percents="unknown ";
                	$state=$ret_codes{UNKNOWN};
        	}

		print "$reverse_ret_codes{$state}\t- $letter ". parse_usage($available_bytes,$capacity_bytes,$usage_percents,$unit);
		$total_state=priority_state($total_state,$state);
	}
	if( $total_state == -1 ) { exit $ret_codes{UNKNOWN}; }
        exit $total_state;
}

sub cpu_usage{
        my $total_state=-1;

        foreach my $v ( read_strings($request{cpu},$opts{n})){
                #data parsing
                unless ( $v =~ m/(.*)\|(\d*)/ ) { error_exit_unknown("ERROR: Error to parse receiving data"); }
                my $cpu=$1;

		my  $usage_percents=$2;
                my  $state=check_alert_state($usage_percents,80,90);

                print "$reverse_ret_codes{$state} - $cpu Usage : $2%\n";
                $total_state=priority_state($total_state,$state);
        }
        if( $total_state == -1 ) { exit $ret_codes{UNKNOWN}; }
        exit $total_state;
}

sub io_disks{
	my $state=$ret_codes{OK};
	my $t_state=-1;

	foreach my $v ( read_strings($request{iodisk})){
                #data parsing
                unless ( $v =~ m/(\d)\|(\d*)\|(.*)/ ) { error_exit_unknown("ERROR: Error to parse receiving data"); }
	
		my $read=$1;
		my $write=$2;
		my $name=$3;	

		if( filter($name,$opts{n})) {
			$state=check_alert_state("$read,$write",50,80);	
	        	print "$reverse_ret_codes{$state} - '$name' '$read' $write\n";
			$t_state=priority_state($t_state,$state);
		}
        }
        if( $t_state == -1 ) { exit $ret_codes{UNKNOWN}; }
	exit $t_state;
}

my %smart_desc=(
        "OK"            => "The element is OK.",
        "Error"         => "An error has occurred.",
        "Degraded"      => "The disk has been degraded.",
        "Unknown"       => "An unknown error has occured.",
        "Pred Fail"     => "The element is functioning nominally but predicting a failure in the near future.",
        "Starting"      => "The element is starting.",
        "Stopping"      => "The element is stopping.",
        "Service"       => "The element is in disk mirror-resilvering, reloading a user permissions list, or other administrative work.",
        "Stressed"      => "The element is functioning, but needs attention. Examples of Stressed states are overload, overheated, and so on.",
        "NonRecover"    => "A non-recoverable error has occurred.",
        "No Contaci"    => "The monitoring system has knowledge of this element, but has never been able to establish communications with it.",
        "Lost Comm"     => "The Managed element is known to exist and has been contacted successfully in the past, but is currently unreachable.",
);

my %smart_states=(
	OK		=> [ "OK", "Starting", "Stopping" ],
	WARNING		=> [ "Degraded", "Unknown", "Pred Fail", "Service", "Stressed", "No Contact"],
	CRITICAL	=> [ "Error", "NonRecover", "Lost Comm" ], 
);

my %smart_alerting=(
	"OK"		=> "OK",
	"Starting"	=> "OK",
	"Stopping"	=> "OK",
	"Degraded"	=> "WARNING",
	"Unknown"	=> "WARNING",
	"Pred Fail"	=> "WARNING",
	"Service"	=> "WARNING",
	"Stressed"	=> "WARNING",
	"No Contact"	=> "WARNING",
	"Error"		=> "CRITICAL",
	"NonRecover"	=> "CRITICAL",
	"Lost Comm"	=> "CRITICAL",
);

sub init_states{
	my ( $state, $values ) = @_;

	foreach my $v ( split(/,/,$values)){
		unless ( exists $smart_alerting{$v} ) { error_exit_unknown("ERROR: The $v state doesn't exist"); }
		$smart_alerting{$v}=$state;
	}
}

sub smart_state{
	init_states( "OK"	, $opts{o} )	if defined $opts{o};
	init_states( "WARNING"	, $opts{w} )	if defined $opts{w};
	init_states( "CRITICAL"	, $opts{c} )	if defined $opts{c};

        my $state=$ret_codes{OK};
        my $t_state=-1;

        foreach my $v ( read_strings($request{smart})){
		unless ( $v =~ m/(.*)\|(.*)\|(.*)/ ) { error_exit_unknown("ERROR: Error to parse receiving data"); }
		my $caption	= $1;
		my $id		= $2;
		my $state	= $3;

		if( filter($id,$opts{n}) ){
			print "$smart_alerting{$state} - ID : $id | Disk : $caption | State : $state => $smart_desc{$state}\n";
			$t_state=priority_state($t_state,$ret_codes{$smart_alerting{$state}});
		}
	}
	if( $t_state == -1 ) { exit $ret_codes{UNKNOWN}; }
        exit $t_state;
}

my %info_request=(
	"services"	=> '"SELECT Caption,Description,PathName,ServiceType,Started,StartMode,State,Status FROM Win32_Service"',
	"disk"		=> '"select CapabilityDescriptions,Caption,Description,DeviceID,FirmwareRevision,Index,Manufacturer,MediaType,Model,Name,Partitions,SerialNumber,Size,Status from Win32_DiskDrive"',
	"os"		=> '"select CSDVersion,InstallDate,LastBootUpTime,LocalDateTime,MUILanguages,NumberOfLicensedUsers,NumberOfProcesses,NumberOfUsers,OSArchitecture,RegisteredUser,SerialNumber,Status FROM Win32_OperatingSystem"',
	"net_perf"	=> '"select CurrentBandwidth,BytesReceivedPerSec,BytesSentPerSec,Name,Frequency_Sys100NS,OutputQueueLength,PacketsReceivedErrors,PacketsReceivedPerSec,PacketsSentPerSec,Timestamp_Sys100NS from Win32_PerfRawData_Tcpip_NetworkInterface"',
	"net_adapter"	=> '"select macaddress,netconnectionID from win32_networkadapter"',
	"net_config"	=> '"Select dnsdomain,dnshostname,ipaddress,ipsubnet,macaddress,description,defaultipgateway,dhcpenabled,dhcpserver,servicename from Win32_NetworkAdapterConfiguration"',
);

sub info_net{
	my @perf=read_strings($info_request{net_perf},".*",0);
	my @adapter=read_strings($info_request{net_adapter},".*",0);
	my @config=read_strings($info_request{net_config},".*",0);
	my @ret;

	my $n=scalar @perf;
	for (my $i=0;$i<$n;$i++){
		print "$adapter[$i] ";
		chomp $adapter[$i];
		chomp $config[$i],
		push @ret,"$adapter[$i]|$config[$i]|$perf[$i]";
	}
	return @ret;
}

my $MAX= 4000;
sub infos{
	my $min ;
	my $max ;
	my $end="false";

	if(defined $opts{l}){
		( $min, $max ) = split(/,/,$opts{l});
	}

	$min //= 0;
	$max //= $min + ( $MAX-1 );

	my $i=0;	

	my @strings ; 
if( $opts{t} eq "net" ) { @strings=info_net(); } 
	else { @strings=read_strings($info_request{$opts{t}},".*",0); }
	foreach my $v ( @strings ){		
		if ( $end eq "true" ){		#On envoit pas plus mais il reste des données
			print "{NOTEND}\n";
			exit 1;
		}

		print "$i.\t$v" if ($i >= $min);
		
		$end = "true" if $i == $max;                 
		$i++;
	}
}

getopt("H:m:n:s:S:hu:w:c:o:t:l:", \%opts);

if	( $opts{m} eq "services" )	{ services_mode();	}
elsif	( $opts{m} eq "ram"	 ) 	{ ram_usage();		}
elsif	( $opts{m} eq "part"	 )	{ part_usage();		} 
elsif 	( $opts{m} eq "cpu"  	 )	{ cpu_usage();		}
elsif	( $opts{m} eq "iodisk"	 )	{ io_disks();		}
elsif   ( $opts{m} eq "smart"	 )	{ smart_state();	}
elsif 	( $opts{m} eq "info"	 )      { infos();		}
else 					{ help();		}
