#!/usr/bin/perl -w
############################## check_speedtest #################
my $Version='1.0';
# Date : Sept 25 2017
# Author : Victor Marin
# Help : esia-sa.com
#################################################################

use Time::HiRes qw(gettimeofday tv_interval);
use strict;
use Getopt::Long;
my $PROGNAME='check_speedtest.pl';
my $TIMEOUT = 15;
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);
my ($opt_h,$opt_V);
my ($hostname);
my $warningDown='20';
my $criticalDown='15';
my $warningUpl='5';
my $criticalUpl='3';
#my @tab_value_read;
my $result;
my $o_warn= undef;    # warning level option
my $o_crit= undef;    # Critical level option


sub print_help() {
  printf "check speedtest.\n";
  print "\n";
  print_usage();
  print "\n";
  print " -w warning value in Mbit/s (default: 20(download),5(upload))\n";
  print " -c critical value in Mbit/s (default: 15(download),3(upload))\n";
  print " -V (--version) Plugin version\n";
  print " -h (--help) usage help (packet => speedtest-cli)\n";
  print "\n";
}

sub print_usage() {
  print "Usage: \n";
  print " $PROGNAME [-w warning] [-c critical]\n";
  print " $PROGNAME [-h | --help]\n";
  print " $PROGNAME [-V | --version]\n";
}

sub isnnum { # Return true if arg is not a number
  my $num = shift;
  if ( $num =~ /^(\d+\.?\d*)|(^\.\d+)$/ ) { return 0 ;}
  return 1;
}

my @priorities=(
                $ERRORS{CRITICAL},
                $ERRORS{WARNING},
                $ERRORS{UNKNOWN},
                $ERRORS{OK});
sub priority_state{
        my($a,$b) = @_;
        if( $a == $b ) { return $a; } #pas besoin d'aller dans la boucles si ils sont identiques

        foreach my $v (@priorities){
                if ( $v == $a || $v == $b ) { return $v; }
        }
}

########## MAIN ##########

&Getopt::Long::config('bundling');
GetOptions(
  "V"   => \$opt_V,        "version"        => \$opt_V,
  "h"   => \$opt_h,        "help"           => \$opt_h,
  "w=s" => \$o_warn,       "warning=s"      => \$o_warn,
  "c=s" => \$o_crit,       "critical=s"     => \$o_crit
);

# -h & --help print help
if ($opt_h) { print_help(); exit $ERRORS{'OK'}; }

if (!defined($o_warn) || !defined($o_crit))
{ 
  print "Put warning and critical values!\n"; print_help(); exit $ERRORS{"UNKNOWN"};
}
# Get rid of % sign
$o_warn =~ s/\%//g; 
$o_crit =~ s/\%//g;
# if -N or -E switch , undef $o_netsnmp
if ( isnnum($o_warn) || isnnum($o_crit)) 
{ 
  print "Numeric value for warning or critical !\n";print_usage(); exit $ERRORS{"UNKNOWN"};
}
if ( ($o_crit != 0) && ($o_warn < $o_crit) ) 
{ 
    print "warning >= critical ! \n";print_usage();exit $ERRORS{"UNKNOWN"};
}
    
my @o_warnL=split(/,/ , $o_warn);
my @o_critL=split(/,/ , $o_crit);
if (($#o_warnL != 1) || ($#o_critL != 1)) 
{ 
  print "2 warnings and critical !\n";print_usage(); exit $ERRORS{"UNKNOWN"}
}
for (my $i=0;$i<2;$i++) 
{
  if ( isnnum($o_warnL[$i]) || isnnum($o_critL[$i])) 
    {
      print "Numeric value for warning or critical !\n";print_usage(); exit $ERRORS{"UNKNOWN"} 
    }
  if (($o_critL[$i]!= 0) && ($o_warnL[$i] < $o_critL[$i]))
  { 
    print "warning >= critical ! \n";print_usage(); exit $ERRORS{"UNKNOWN"}
  }
}
$warningDown=$o_warnL[0];
$criticalDown=$o_critL[0];
$warningUpl=$o_warnL[1];
$criticalUpl=$o_critL[1];

my $return=$ERRORS{OK};

my $t0 = [gettimeofday];
my $output = `speedtest --simple`;
my $interval = sprintf("%.2f",tv_interval($t0)); # interval secondes

my $ping = ($output =~ /Ping:\s+([0-9]*[\,\.][0-9]*)\s(\S+)/); 
$ping=$1;
my $unityPing = $2;

my $download = ($output =~ /Download:\s+([0-9]*[\,\.][0-9]*)\s+(\S+)/);
$download=$1;
my $unitDown=$2;

my $upload = ($output =~ /Upload:\s+([0-9]*[\,\.][0-9]*)\s+(\S+)/);
$upload=$1;
my $unitUpl=$2;

if(!defined($ping)||!defined($download)||!defined($upload) ||!defined($interval))
{
  print "Verify your command \n";
  exit($ERRORS{"WARNING"});
}
else
{
  #Ping
  print ("Ping : $ping $unityPing \n");
  print ("Runtime : $interval secondes\n");

  #Download
  if($download < $criticalDown)
  {
    print ("CRITICAL : Download :$download $unitDown (< $criticalDown Mbit/s) \n");
    $return = priority_state($return,$ERRORS{CRITICAL}) 
  }
  else
  {
    if($download < $warningDown)
    {
      print ("WARNING : Download : $download $unitDown ($warningDown Mbit/s > X > $criticalDown Mbit/s) \n");
      $return = priority_state($return,$ERRORS{WARNING}) 
    }
    else
    {
      print ("OK : Download : $download $unitDown (> $warningDown Mbit/s) \n");      
    }
  }

  #Upload
  if($upload < $criticalUpl)
  {
    print ("CRITICAL : Upload : $upload $unitUpl (< $criticalUpl Mbit/s) |upl=$upload,down=$download,warningUpl=$warningUpl,criticalUpl=$criticalUpl,warningDown=$warningDown,criticalDown=$criticalDown,ping=$ping,time=$interval \n");
   $return = priority_state($return,$ERRORS{CRITICAL}) 
  }
  else
  {
    if($upload < $warningUpl)
    {
      print ("WARNING : Upload : $upload $unitUpl ($warningUpl Mbit/s > X > $criticalUpl Mbit/s) |upl=$upload,down=$download,warningUpl=$warningUpl,criticalUpl=$criticalUpl,warningDown=$warningDown,criticalDown=$criticalDown,ping=$ping,time=$interval \n");
     $return = priority_state($return,$ERRORS{WARNING}) 
    }
    else
    {
      print ("OK : Upload : $upload $unitUpl (> $warningUpl Mbit/s) |upl=$upload,down=$download,warningUpl=$warningUpl,criticalUpl=$criticalUpl,warningDown=$warningDown,criticalDown=$criticalDown,ping=$ping,time=$interval \n");      
    }
  }
}
exit $return;